;;;
;;; ipc.h
;;;
;;; created: <Fri May  7 21:31:43 1999>
;;; last modified: <Sun Jul  2 23:16:52 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the ipc functions in libASM.a
;;;

%ifndef _ipc_h_
%define _ipc_h_

;;;*
;;;* PipeIN
;;;*
;;;*  In: PipeIN *filename, **argv, **envp
;;;*      (*filename [eax+0], **argv [eax+4], **envp [eax+8], dummy (2 dwords)
;;;*       [eax+12])
;;;*  Out: eax=fd
;;;*
;;;*  Set up a pipe: Read childs output (STDOUT) from fd.
;;;*
%macro PipeIN 1-3
	extern _PipeIN
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `PipeIN' expects 1 or 3 parameters."
%elif %0 = 3
	sub esp,byte 8		; dummy
	__push %3, %2, %1
	mov eax,esp
%endif
	call _PipeIN
%if %0 = 3
	add esp,byte 12+8
%endif
%endm

;;;*
;;;* PipeIn
;;;*
;;;*  In: PipeIn 'progname', 'arg1', 'arg2', ...
;;;*  Out: eax=fd
;;;*
;;;*  Set up a pipe: Read childs output (STDOUT) from fd.
;;;*
%macro PipeIn 1-*
	section .data
%%data:
	db 0
%rep %0
	db %1,0
%rotate 1
%endrep
%%argv:
%rep %0
	dd 0
%endrep
	dd 0
	section .text
	pushad
	mov ecx,%0
	mov esi,%%argv
	mov edi,%%data
%%_loop:
	push ecx
	mov ecx,0xffffffff
	mov al,0
	cld
	repne scasb
	mov [esi],edi
	add esi,byte 4
	pop ecx
	loop %%_loop
	popad
	PipeIN dword %%data+1, dword %%argv, dword [_envp]
%endm

;;;*
;;;* PipeOUT
;;;*
;;;*  In: PipeOUT *filename, **argv, **envp
;;;*      (*filename [eax+0], **argv [eax+4], **envp [eax+8], dummy (2 dwords)
;;;*       [eax+12])
;;;*  Out: eax=fd
;;;*
;;;*  Set up a pipe: Writes data to childs STDIN using fd.
;;;*
%macro PipeOUT 1-3
	extern _PipeOUT
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `PipeOUT' expects 1 or 3 parameters."
%elif %0 = 3
	sub esp,byte 8		; dummy
	__push %3, %2, %1
	mov eax,esp
%endif
	call _PipeOUT
%if %0 = 3
	add esp,byte 12+8
%endif
%endm

;;;*
;;;* PipeOut
;;;*
;;;*  In: PipeOut 'progname', 'arg1', 'arg2', ...
;;;*  Out: eax=fd
;;;*
;;;*  Set up a pipe: Writes data to childs STDIN using fd.
;;;*
%macro PipeOut 1-*
	section .data
%%data:
	db 0
%rep %0
	db %1,0
%rotate 1
%endrep
%%argv:
%rep %0
	dd 0
%endrep
	dd 0
	section .text
	pushad
	mov ecx,%0
	mov esi,%%argv
	mov edi,%%data
%%_loop:
	push ecx
	mov ecx,0xffffffff
	mov al,0
	cld
	repne scasb
	mov [esi],edi
	add esi,byte 4
	pop ecx
	loop %%_loop
	popad
	PipeOUT dword %%data+1, dword %%argv, dword [_envp]
%endm

;;;*
;;;* PipeIO
;;;*
;;;*  In: PipeIO *fd, *filename, **argv, **envp
;;;*      (*fd [eax+0], *filename [eax+4], **argv [eax+8], **envp [eax+12],
;;;*       dummy (4 dwords) [eax+16])
;;;*  Out: eax=*fd
;;;*           [fd+0]: fd to read from
;;;*           [fd+4]: fd to write to
;;;*
;;;*  Set up a pipe: Writes/reads data to/from childs STDIN/STDOUT using 
;;;*  *fd.
;;;*
%macro PipeIO 1-4
	extern _PipeIO
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	sub esp,byte 16		; dummy
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `PipeIO' expects 1 or 4 parameters."
%endif
	call _PipeIO
%if %0 = 4
	add esp,byte 16+16
%endif
%endm

;;;*
;;;* PipeIo
;;;*
;;;*  In: PipeIo *fd, 'progname', 'arg1', 'arg2', ...
;;;*  Out: eax=*fd
;;;*           [fd+0]: fd to read from
;;;*           [fd+4]: fd to write to
;;;*
;;;*  Set up a pipe: Writes/reads data to/from childs STDIN/STDOUT using 
;;;*  *fd.
;;;*
%macro PipeIo 2-*
	section .data
%%data:
	db 0
%rep %0-1
	db %2,0
%rotate 1
%endrep
%%argv:
%rep %0-1
	dd 0
%endrep
%rotate 1
	dd 0
	section .text
	pushad
	mov ecx,%0-1
	mov esi,%%argv
	mov edi,%%data
%%_loop:
	push ecx
	mov ecx,0xffffffff
	mov al,0
	cld
	repne scasb
	mov [esi],edi
	add esi,byte 4
	pop ecx
	loop %%_loop
	popad
	PipeIO %1, dword %%data+1, dword %%argv, dword [_envp]
%endm


;;;*
;;;* shmget
;;;*
;;;*  In: shmget key, size, shmflg
;;;*      (key [eax+0], size [eax+4], shmflg [eax+8])
;;;*  Out: eax=shmid (on success), -1 (on error)
;;;*
;;;*  See shmget(2).
;;;*
%macro shmget 1-3
	extern _shmget
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%else
 %error "Macro `shmget' expects 1 or 3 parameters."
%endif
	call _shmget
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* shmctl
;;;*
;;;*  In: shmctl shmid, cmd, *buf
;;;*      (shmid [eax+0], cmd [eax+4], *buf [eax+8])
;;;*  Out: eax=0 (on success), -1 (on error)
;;;*
;;;*  See shmctl(2).
;;;*
%macro shmctl 1-3
	extern _shmctl
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%else
 %error "Macro `shmctl' expects 1 or 3 parameters."
%endif
	call _shmctl
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* shmat
;;;*
;;;*  In: shmat shmid, *shmaddr, shmflg
;;;*      (shmid [eax+0], *shmaddr [eax+4], shmflg [eax+8])
;;;*  Out: eax=address of the attached shm seg (on success), -1 (on error)
;;;*
;;;*  See shmat(2).
;;;*
%macro shmat 1-3
	extern _shmat
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%else
 %error "Macro `shmat' expects 1 or 3 parameters."
%endif
	call _shmat
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* shmdt
;;;*
;;;*  In: shmdt *shmaddr (eax)
;;;*  Out: eax=0 (on success), -1 (on error)
;;;*
;;;*  See shmdt(2).
;;;*
%macro shmdt 1
	extern _shmat
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
	call _shmdt
%endm


%endif	; _ipc_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: