;;;
;;; maths.h
;;;
;;; created: <Thu Feb 24 22:29:39 2000>
;;; last modified: <Thu Apr 27 20:17:21 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This contains some formulas for the FPU.
;;;

%ifndef _maths_h_
%define _maths_h_

;;;*
;;;* m_log10x: Compute st0 = log10(st0)
;;;*
;;;*  loga(u) = loga(b) * logb(u)
;;;*  log10(x) = log10(2) * log2(x)
;;;*
%macro m_log10x 0
	fldlg2
	fxch st1
	fyl2x
%endm


;;;*
;;;* m_logex: Compute st0 = loge(st0)
;;;*
;;;*  loga(u) = loga(b) * logb(u)
;;;*  ln(x) = ln(2) * log2(x)
;;;*
%macro m_logex 0
	fldln2
	fxch st1
	fyl2x
%endm


;;;*
;;;* m_exp10: Compute st0 = 10^st0
;;;*
;;;*  x^r = (a^loga(x))^r = a^(r * loga(x))
;;;*  10^x = 2^(x*log2(10))
;;;*
%macro m_exp10 0
	fldl2t
	fmulp st1
	fld st0
	frndint
	fsub st1,st0
	fxch st1
	f2xm1
	fld1
	faddp st1
	fscale
	fstp st1
%endm


;;;*
;;;* m_expe: Compute st0 = e^st0
;;;*
;;;*  x^r = (a^loga(x))^r = a^(r * loga(x))
;;;*  e^x = 2^(x*log2(e))
;;;*
%macro m_expe 0
	fldl2e
	fmulp st1
	fld st0
	frndint
	fsub st1,st0
	fxch st1
	f2xm1
	fld1
	faddp st1
	fscale
	fstp st1
%endm

%endif	; _maths_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: