;;;
;;; misc.h
;;;
;;; created: <Fri May  7 21:10:17 1999>
;;; last modified: <Thu Oct 24 23:35:49 2002>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the misc functions in libASM.a
;;;

%ifndef _misc_h_
%define _misc_h_

;;;*
;;;* StartPoint
;;;*
;;;*  In: StartPoint
;;;*  Out: -
;;;*
;;;*  Define entrypoint for program
;;;*
%macro StartPoint 0
	extern _StartupCode,_envp,_argv,_argp,_errno,_kver
	global __entry
	section .text
__entry:
	call _StartupCode
%endm


;;;*
;;;* GetCPUType
;;;*
;;;*  In: GetCPUType *CPUInfo (eax)
;;;*  Out: dword [CPUInfo.ProcessorType]
;;;*		0 (386), 1 (486), 2 (586), ...
;;;*       dword [CPUInfo.CPUExtensions]
;;;*		CPU Extensions
;;;*			CPUInfo_HaveFPU: FPU
;;;*			CPUInfo_HaveCPUID: cpuid supported
;;;*			CPUInfo_HaveMMX: MMX
;;;*			CPUInfo_Have3DNow: 3DNow!
;;;*			CPUInfo_HaveCMPXCHG8B: cmpxchg8b supported
;;;*			CPUInfo_HaveCondMOV: conditional move instructions
;;;*			CPUInfo_Have3DNowExt: 3DNowExt
;;;*       4 dwords [CPUInfo.VendorString]
;;;*		vendor string (12 bytes+4 bytes NULL)
;;;*
;;;*  Get type of CPU and more infos
;;;*
%macro GetCPUType 1
	extern _GetCPUType
%ifnidni %1,eax
	mov eax,%1
%endif
	call _GetCPUType
%endm

%define CPUInfo_HaveFPU 1
%define CPUInfo_HaveCPUID 2
%define CPUInfo_HaveMMX 4
%define CPUInfo_Have3DNow 8
%define CPUInfo_HaveCMPXCHG8B 16
%define CPUInfo_HaveCondMOV 32
%define CPUInfo_Have3DNowExt 64
%define CPUInfo_HaveMMXExt 128

struc CPUInfo
.ProcessorType:		resd 1
.CPUExtensions:		resd 1
.VendorString:		resd 4
endstruc


;;;*
;;;* Assert
;;;*
;;;*  In: Assert val
;;;*  Out: -
;;;*
;;;*  Test val, exit program and give message (if defined) if val=0
;;;*
%macro Assert 1
	extern _Assert
	section .data
%define Assert_Gaense "
;;;"
%%data:	db Assert_Gaense %1 Assert_Gaense,13,10,0 ; This is not nice, but it works
%ifndef WriteDebugInfo_FILE_DEFINED
%define WriteDebugInfo_FILE_DEFINED
WriteDebugInfo_FILE: db __FILE__,0
%endif
	section .text
	__push dword __LINE__, dword WriteDebugInfo_FILE, dword %%data, %1
	mov eax,esp
	call _Assert
	add esp,byte 16
%endm

;;;*
;;;* Random
;;;*
;;;*  In: Random n
;;;*  Out: eax=random
;;;*
;;;*  Read n (n=1..4) bytes of random from /dev/random. Make sure, that this
;;;*  device is initialised for 'real' randomness...
;;;*
%macro Random 1
	extern _Random
%ifnidni %1,eax
	mov eax,%1
%endif
	call _Random
%endm

;;;*
;;;* RandomString
;;;*
;;;*  In: RandomString *buf, n (*buf [eax+0], n [eax+4])
;;;*  Out: eax=*buf
;;;*
;;;*  Read n bytes of random from /dev/random to *buf. Make sure, that this
;;;*  device is initialised for 'real' randomness...
;;;*
%macro RandomString 2
	extern _RandomString
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _RandomString
%if %0 = 2
	add esp,byte 8
%endif
%endm

%endif	; _misc_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End:
