;;;
;;; x_func.h
;;;
;;; created: <Wed Mar  8 21:35:43 2000>
;;; last modified: <Sat Sep  9 20:37:24 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the x functions in libASM.a
;;;

%ifndef _x_func_h_
%define _x_func_h_


;;;*
;;;* XOpenDisplay
;;;*
;;;*  In: XOpenDisplay *display (eax)
;;;*  Out: eax=*dpy
;;;*
;;;*  Connect to Xserver.
;;;*  (If *display=0 then the environment variable 'DISPLAY' is used)
;;;*
%macro XOpenDisplay 0-1
	extern _XOpenDisplay
%if %0 = 0
	mov eax,0
%else
 %ifnidni %1,eax
	mov eax,%1
 %endif
%endif
	call _XOpenDisplay
%endm


;;;*
;;;* XCloseDisplay
;;;*
;;;*  In: XCloseDisplay *dpy (eax)
;;;*  Out: eax=0 (on success); eax=-1 (on error)
;;;*
;;;*  Disconnects from Xserver.
;;;*
%macro XCloseDisplay 1
	extern _XCloseDisplay
%ifnidni %1,eax
	mov eax,%1
%endif
	call _XCloseDisplay
%endm


;;;*
;;;* XAllocID
;;;*
;;;*  In: XAllocID *dpy (eax)
;;;*  Out: eax=resource ID
;;;*
;;;*  Resource ID allocation routine.
;;;*
%macro XAllocID 1
	extern _XAllocID
%ifnidni %1,eax
	mov eax,%1
%endif
	call _XAllocID
%endm


;;;*
;;;* XSync
;;;*
;;;*  In: XSync *dpy, *buf, len (*dpy [eax+0], *buf [eax+4], len [eax+8])
;;;*  Out: eax=0 (on success) !=0 (reply: memory chunk)
;;;*
;;;*  Flush buffer and check for replys, errors and events.
;;;*
%macro XSync 1-3
	extern _XSync
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `XSync' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _XSync
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* XScreenAddr
;;;*
;;;*  In: XScreenAddr *dpy, screen (*dpy [eax+0], screen [eax+4])
;;;*  Out: eax=addr (on success); -1 (on error)
;;;*
;;;*  Return address of screenstruc screen.
;;;*  (Called ScreenOfDisplay X11-proto)
;;;*
%macro XScreenAddr 2
	extern _XScreenAddr
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XScreenAddr
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XDepthAddr
;;;*
;;;*  In: XDepthAddr *dpy, depth, screen
;;;*      (*dpy [eax+0], depth [eax+4], screen [eax+8])
;;;*  Out: eax=addr (on success); -1 (on error)
;;;*
;;;*  Return address of depthstruc with depth depth in screen.
;;;*
%macro XDepthAddr 1-3
	extern _XDepthAddr
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `XDepthAddr' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _XDepthAddr
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* XVisualTypeIDAddr
;;;*
;;;*  In: XVisualIDTypeAddr *dpy, vid
;;;*      (*dpy [eax+0], vid [eax+4])
;;;*  Out: eax=addr (on success); -1 (on error)
;;;*
;;;*  Return address of visualtype VisualID struc.
;;;*
%macro XVisualTypeIDAddr 1-2
	extern _XVisualTypeIDAddr
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XVisualTypeIDAddr
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XFindVisualTypeDSAddr
;;;*
;;;*  In: XFindVisualDSTypeAddr *dpy, vid, depth, screen
;;;*      (*dpy [eax+0], vid [eax+4], depth [eax+8], screen [eax+12])
;;;*  Out: eax=addr (on success); -1 (on error)
;;;*
;;;*  Return address of visualtype VisualID struc with depth in screen.
;;;*
%macro XFindVisualTypeDSAddr 1-4
	extern _XFindVisualTypeDSAddr
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XFindVisualTypeDSAddr' expects 1 or 4 parameters."
%endif
	call _XFindVisualTypeDSAddr
%if %0 = 4
	add esp,byte 16
%endif
%endm


;;;*
;;;* XFindVisualTypeSAddr
;;;*
;;;*  In: XVisualTypeSAddr *dpy, vid, screen
;;;*      (*dpy [eax+0], vid [eax+4], screen[eax+8])
;;;*  Out: eax=addr (on success); -1 (on error)
;;;*
;;;*  Return address of visualtype VisualID struc in screen.
;;;*
%macro XFindVisualTypeSAddr 1-3
	extern _XFindVisualTypeSAddr
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `XFindVisualTypeSAddr' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _XFindVisualTypeSAddr
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* XGetScanlinePad
;;;*
;;;*  In: XGetScanlinePad *dpy, depth
;;;*      (*dpy [eax+0], depth [eax+4])
;;;*  Out: eax=pad (on success); -1 (on error)
;;;*
;;;*  Return scanline pad for depth.
;;;*
%macro XGetScanlinePad 1-2
	extern _XGetScanlinePad
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XGetScanlinePad
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XGetBitsPerPixel
;;;*
;;;*  In: XGetBitsPerPixel *dpy, depth
;;;*      (*dpy [eax+0], depth [eax+4])
;;;*  Out: eax=bits per pixel (on success); -1 (on error)
;;;*
;;;*  Return bits per pixel for depth.
;;;*
%macro XGetBitsPerPixel 1-2
	extern _XGetBitsPerPixel
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XGetBitsPerPixel
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XCreateWindow
;;;*
;;;*  In: XCreateWindow *dpy, parent, x, y, width, height, borderWidth, depth,
;;;*      class, *visual, valuemask, *attributes
;;;*      (*dpy [eax+0], parent [eax+4], x [eax+8], y [eax+12], width [eax+16],
;;;*       height [eax+20], borderWidth [eax+24], depth [eax+28],
;;;*       class [eax+32], *visual [eax+36], valuemask [eax+40],
;;;*       *attributes [eax+44])
;;;*  Out: eax=wid
;;;*
;;;*  Create new window an assign the identifier wid with it.
;;;*
%macro XCreateWindow 1-12
	extern _XCreateWindow
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 12
	__push %12, %11, %10, %9, %8, %7, %6, %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XCreateWindow' expects 1 or 12 parameters."
%endif
	call _XCreateWindow
%if %0 = 12
	add esp,byte 48
%endif
%endm



;;;*
;;;* XChangeWindowAttributes
;;;*
;;;*  In: XChangeWindowAttributes *dpy, wid, valuemask, *attributes
;;;*      (*dpy [eax+0], wid [eax+4], valuemask [eax+8], *attributes [eax+12])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Change window attributes.
;;;*
%macro XChangeWindowAttributes 1-4
	extern _XChangeWindowAttributes
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XChangeWindowAttributes' expects 1 or 4 parameters."
%endif
	call _XChangeWindowAttributes
%if %0 = 4
	add esp,byte 16
%endif
%endm


;;;*
;;;* XDestroyWindow
;;;*
;;;*  In: XDestroyWindow *dpy, wid (*dpy [eax+0], wid [eax+4])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk)
;;;*
%macro XDestroyWindow 1-2
	extern _XDestroyWindow
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XDestroyWindow
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XDestroySubwindows
;;;*
;;;*  In: XDestroySubwindows *dpy, wid (*dpy [eax+0], wid [eax+4])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk)
;;;*
%macro XDestroySubwindows 1-2
	extern _XDestroySubwindows
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XDestroySubwindows
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XReparentWindow
;;;*
;;;*  In: XReparentWindow *dpy, wid, parent, x, y
;;;*      (*dpy [eax+0], wid [eax+4], parent [eax+8], x [eax+12], y [eax+16])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Change parent, x and y of wid.
;;;*
%macro XReparentWindow 1-5
	extern _XReparentWindow
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 5
	__push %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XReparentWindow' expects 1 or 5 parameters."
%endif
	call _XReparentWindow
%if %0 = 5
	add esp,byte 20
%endif
%endm


;;;*
;;;* XMapWindow
;;;*
;;;*  In: XMapWindow *dpy, wid (*dpy [eax+0], wid [eax+4])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk)
;;;*
;;;*  Map window on display.
;;;*
%macro XMapWindow 1-2
	extern _XMapWindow
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XMapWindow
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XMapSubwindows
;;;*
;;;*  In: XMapSubwindows *dpy, wid (*dpy [eax+0], wid [eax+4])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk)
;;;*
;;;*  Map subwindows on display.
;;;*
%macro XMapSubwindows 1-2
	extern _XMapSubwindows
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XMapSubwindows
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XUnmapWindow
;;;*
;;;*  In: XUnmapWindow *dpy, wid (*dpy [eax+0], wid [eax+4])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk)
;;;*
;;;*  Unmap window from display
;;;*
%macro XUnmapWindow 1-2
	extern _XUnmapWindow
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XUnmapWindow
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XUnmapSubwindows
;;;*
;;;*  In: XUnmapSubwindows *dpy, wid (*dpy [eax+0], wid [eax+4])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk)
;;;*
;;;*  Unmap subwindows from display
;;;*
%macro XUnmapSubwindows 1-2
	extern _XUnmapSubwindows
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XUnmapSubwindows
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;*
;;;* XChangeProperty
;;;*
;;;*  In: XChangeProperty *dpy, wid, property, type, format, mode, *data, nelements
;;;*      (*dpy [eax+0], wid [eax+4], property [eax+8], type [eax+12]
;;;*       format [eax+16] , mode [eax+20], *data [eax+24], nelements [eax+28])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk); -1 (on error)
;;;*
;;;*  Change property of wid
;;;*
%macro XChangeProperty 1-8
	extern _XChangeProperty
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 8
	__push %8, %7, %6, %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XChangeProperty' expects 1 or 8 parameters."
%endif
	call _XChangeProperty
%if %0 = 8
	add esp,byte 32
%endif
%endm


;;;*
;;;* XSetWMName
;;;*
;;;*  In: XSetWMName *dpy, wid, encoding, format, *value, nitems
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk); -1 (on error)
;;;*
;;;*  Set name of window.
;;;*  To set windowname: encoding=XA_STRING, format=8, *value=ASCIIZ,
;;;*  nitems=Strlen(ASCIIZ)
%macro XSetWMName 6
	XChangeProperty %1, %2, dword XA_WM_NAME, %3, %4, dword PropModeReplace, %5, %6
%endm


;;;*
;;;* XSetWMIconName
;;;*
;;;*  In: XSetWMIconName *dpy, wid, encoding, format, *value, nitems
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk); -1 (on error)
;;;*
;;;*  Set the name to be displayed an a window's icon.
;;;*  encoding=XA_STRING, format=8, *value=ASCIIZ, nitems=Strlen(ASCIIZ)
;;;*
%macro XSetWMIconName 6
	XChangeProperty %1, %2, dword XA_WM_ICON_NAME, %3, %4, dword PropModeReplace, %5, %6
%endm


;;;*
;;;* XSetWMClientMachine
;;;*
;;;*  In: XSetWMClientMachine dpy, wid, encoding, format, *value, nitems
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk); -1 (on error)
;;;*
;;;*  encoding=XA_STRING, format=8, *value=ASCIIZ, nitems=Strlen(ASCIIZ)
;;;*
%macro XSetWMClientMachine 6
	XChangeProperty %1, %2, dword XA_WM_CLIENT_MACHINE, %3, %4, dword PropModeReplace, %5, %6
%endm


;;;*
;;;* XDeleteProperty
;;;*
;;;*  In: XDeleteProperty *dpy, wid, property
;;;*      (*dpy [eax+0], wid [eax+4], property [eax+8])
;;;*  Out: eax=0 (on success); !=0 (reply: memory chunk)
;;;*
;;;*  Delete property of wid.
;;;*
%macro XDeleteProperty 1-3
	extern _XDeleteProperty
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
 %error "Macro `XDeleteProperty' expects 1 or 3 parameters."
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%endif
	call _XDeleteProperty
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* XGetProperty
;;;*
;;;*  In: XGetProperty *dpy, wid, property, offset, length, delete, type,
;;;*                 *actual_type, *actual_format, *nitems, *bytesafter, *value
;;;*      (*dpy [eax+0], wid [eax+4], property [eax+8], offset [eax+12],
;;;*       length [eax+16], delete [eax+20], type [eax+24], *actual_type [eax+28],
;;;*       *actual_format [eax+32], *nitems [eax+36], *bytesafter [eax+40],
;;;*       *value [eax+44])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Get property. Fills in *actual_type, *actual_format, *nitems,
;;;*  *bytesafter and *value.
;;;*
%macro XGetProperty 1-12
	extern _XGetProperty
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 12
	__push %12, %11, %10, %9, %8, %7, %6, %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XGetProperty' expects 1 or 12 parameters."
%endif
	call _XGetProperty
%if %0 = 12
	add esp,byte 48
%endif
%endm


;;;*
;;;* XGetWMName
;;;*
;;;*  In: XGetWMName *dpy, wid, *actual_type, *actual_format, *nitems,
;;;*      *bytesafter, *value
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Get window name.
;;;*
%macro XGetWMName 7
	XGetProperty %1, %2, dword XA_WM_NAME, dword 0, dword 1000000, dword False, dword AnyPropertyType, %3, %4, %5, %6, %7
%endm


;;;*
;;;* XGetWMIconName
;;;*
;;;*  In: XGetWMIconName *dpy, wid, *actual_type, *actual_format, *nitems,
;;;*      *bytesafter, *value
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Get window icon name.
;;;*
%macro XGetWMIconName 7
	XGetProperty %1, %2, dword XA_WM_ICON_NAME, dword 0, dword 1000000, dword False, dword AnyPropertyType, %3, %4, %5, %6, %7
%endm


;;;*
;;;* XGetWMClientMachine
;;;*
;;;*  In: XGetWMClientMachine *dpy, wid, *actual_type, *actual_format, *nitems,
;;;*      *bytesafter, *value
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
%macro XGetWMClientMachine 7
	XGetProperty %1, %2, dword XA_WM_CLIENT_MACHINE, dword 0, dword 1000000, dword False, dword AnyPropertyType, %3, %4, %5, %6, %7
%endm


;;;*
;;;* XCreateGC
;;;*
;;;*  In: XCreateGC *dpy, wid, valuemask, *values
;;;*      (*dpy [eax+0], wid [eax+4], valuemask [eax+8], *values [eax+12])
;;;*  Out: eax=cid (on success); =-1 (on error)
;;;*
;;;*  Create grafics context (cid).
;;;*
%macro XCreateGC 1-4
	extern _XCreateGC
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XCreateGC' expects 1 or 4 parameters."
%endif
	call _XCreateGC
%if %0 = 4
	add esp,byte 16
%endif
%endm


;;;*
;;;* XChangeGC
;;;*
;;;*  In: XChangeGC *dpy, gc, valuemask, *values
;;;*      (*dpy [eax+0], gc [eax+4], valuemask [eax+8], *values [eax+12])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Change grafics context (gc).
;;;*
%macro XChangeGC 1-4
	extern _XChangeGC
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XChangeGC' expects 1 or 4 parameters."
%endif
	call _XChangeGC
%if %0 = 4
	add esp,byte 16
%endif
%endm


;;;*
;;;* XCopyGC
;;;*
;;;*  In: XCopyGC *dpy, src_gc, valuemask, dst_gc
;;;*      (*dpy [eax+0], src_gc [eax+4], valuemask [eax+8], dst_gc [eax+12])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Copy grafics context (src_gc->dst_gc).
;;;*
%macro XCopyGC 1-4
	extern _XCopyGC
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XCopyGC' expects 1 or 4 parameters."
%endif
	call _XCopyGC
%if %0 = 4
	add esp,byte 16
%endif
%endm


;;;*
;;;* XFreeGC
;;;*
;;;*  In: XFreeGC *dpy, gc
;;;*      (*dpy [eax+0], gc [eax+4])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Free grafics context (gc).
;;;*
%macro XFreeGC 1-2
	extern _XFreeGC
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%else
	__push %2, %1
	mov eax,esp
%endif
	call _XFreeGC
%if %0 = 4
	add esp,byte 8
%endif
%endm


;;;*
;;;* XClearArea
;;;*
;;;*  In: XClearArea *dpy, wid, x, y, width, height, exposures
;;;*      (*dpy [eax+0], wid [eax+4], x [eax+8], y [eax+12], width [eax+16]
;;;*       height [eax+20], exposures [eax+24])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Clear area in window.
;;;*
%macro XClearArea 1-7
	extern _XClearArea
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 7
	__push %7, %6, %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XClearArea' expects 1 or 7 parameters."
%endif
	call _XClearArea
%if %0 = 7
	add esp,byte 28
%endif
%endm


;;;*
;;;* XClearWindow
;;;*
;;;*  In: XClearWindow *dpy, wid
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Clear window.
;;;*
%macro XClearWindow 2
;;; This values mean to clear the entire window
	XClearArea %1, %2, dword 0, dword 0, dword 0, dword 0, dword xFalse
%endm


;;;*
;;;* XPolyPoint
;;;*
;;;*  In: XPolyPoint *dpy, wid, gc, cmode, npoints, *points
;;;*      (*dpy [eax+0], wid [eax+4], gc [eax+8], cmode [eax+12],
;;;*       npoints [eax+16], *points [eax+20])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Draw points in wid. Use gc to create the point's context. cmode specifies
;;;*  the coordinate mode: CoordModeOrigin (relative to the origin) and
;;;*  CoordModePrevious (relative to previous point).
;;;*
%macro XPolyPoint 1-6
	extern _XPolyPoint
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 6
	__push %6, %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XPolyPoint' expects 1 or 6 parameters."
%endif
	call _XPolyPoint
%if %0 = 6
	add esp,byte 24
%endif
%endm


;;;*
;;;* XQueryExtension
;;;*
;;;*  In: XQueryExtension *dpy, *name, *mopcode, *firstevent, *firsterror
;;;*      (*dpy [eax+0], name [eax+4], *mopcode [eax+8], *firstevent [eax+12]
;;;*       *firsterror [eax+16])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Query for extension. Return [mopcode], [firstevent], [firsterror].
;;;*
%macro XQueryExtension 1-5
	extern _XQueryExtension
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 5
	__push %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XQueryExtension' expects 1 or 5 parameters."
%endif
	call _XQueryExtension
%if %0 = 5
	add esp,byte 20
%endif
%endm


;;;*
;;;* XCMiscGetVersion
;;;*
;;;*  In: XCMiscGetVersion *dpy, client_major_version, client_minor_version,
;;;*                       *server_major_version, *server_minor_version
;;;*      (*dpy [eax+0], client_major_version [eax+4],
;;;*       client_minor_version [eax+8], *server_minor_version [eax+12],
;;;*       *server_minor_version [eax+16])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Get and set XC-Misc Extension protocol version.
;;;*
%macro XCMiscGetVersion 1-5
	extern _XCMiscGetVersion
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 5
	__push %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XCMiscGetVersion' expects 1 or 5 parameters."
%endif
	call _XCMiscGetVersion
%if %0 = 5
	add esp,byte 20
%endif
%endm


;;;*
;;;* XCMiscGetXIDRange
;;;*
;;;*  In: XCMiscGetXIDRange *dpy, *start_id, *count
;;;*      (*dpy [eax+0], *start_id [eax+4], *count [eax+8])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Get next XID and range.
;;;*
%macro XCMiscGetXIDRange 1-3
	extern _XCMiscGetXIDRange
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 3
	__push %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XCMiscGetXIDRange' expects 1 or 3 parameters."
%endif
	call _XCMiscGetXIDRange
%if %0 = 3
	add esp,byte 12
%endif
%endm


;;;*
;;;* XCMiscGetXIDList
;;;*
;;;*  In: XCMiscGetXIDList *dpy, count, *ids, *nids
;;;*      (*dpy [eax+0], count [eax+4], *ids [eax+8], *nids [eax+12])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Get list of IDs (requested length: count, returned length: nids).
;;;*
%macro XCMiscGetXIDList 1-4
	extern _XCMiscGetXIDList
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XCMiscGetXIDList' expects 1 or 4 parameters."
%endif
	call _XCMiscGetXIDList
%if %0 = 4
	add esp,byte 16
%endif
%endm


;;;*
;;;* XShmQueryVersion
;;;*
;;;*  In: XShmQueryVersion *dpy, *majorVersion, *minorVersion, *sharedPixmaps
;;;*      (*dpy [eax+0], *majorVersion [eax+4], *minorVersion [eax+8],
;;;*       *sharedPixmaps [eax+12])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Get version of MIT-SHM Extension.
;;;*
%macro XShmQueryVersion 1-4
	extern _XShmQueryVersion
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 4
	__push %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XShmQueryVersion' expects 1 or 4 parameters."
%endif
	call _XShmQueryVersion
%if %0 = 4
	add esp,byte 16
%endif
%endm


;;;*
;;;* XShmCreateImage
;;;*
;;;*  In: XShmCreateImage *dpy, *image, *visual, depth, format, *data,
;;;*                      *shminfo, width, height
;;;*      (*dpy [eax+0], *image [eax+4], *visual [eax+8], depth [eax+12],
;;;*       format [eax+16], *data [eax+20], *shminfo [eax+24], width [eax+28],
;;;*       height [eax+32])
;;;*  Out: eax=*image (on success); =-1 (on error)
;;;*
;;;*  Create ximagestruc.
;;;*
%macro XShmCreateImage 1-9
	extern _XShmCreateImage
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 9
	__push %9, %8, %7, %6, %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XShmCreateImage' expects 1 or 9 parameters."
%endif
	call _XShmCreateImage
%if %0 = 9
	add esp,byte 36
%endif
%endm


;;;*
;;;* XShmAttach
;;;*
;;;*  In: XShmAttach *dpy, *shminfo
;;;*      (*dpy [eax+0], *shminfo [eax+4])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Attach shm to X server..
;;;*
%macro XShmAttach 1-2
	extern _XShmAttach
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XShmAttach
%if %0 = 9
	add esp,byte 8
%endif
%endm


;;;*
;;;* XShmDetach
;;;*
;;;*  In: XShmDetach *dpy, *shminfo
;;;*      (*dpy [eax+0], *shminfo [eax+4])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Detach shm from X server..
;;;*
%macro XShmDetach 1-2
	extern _XShmDetach
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _XShmDetach
%if %0 = 9
	add esp,byte 8
%endif
%endm


;;;*
;;;* XShmPutImage
;;;*
;;;*  In: XShmPutImage *dpy, wid, gc, *image, src_x, src_y, dst_x, dst_y,
;;;*                   src_width, src_height, send_event
;;;*      (*dpy [eax+0], wid [eax+4], gc [eax+8], *image [eax+12],
;;;*       src_x [eax+16], src_y [eax+20], dst_x [eax+24], dst_y [eax+28],
;;;*       src_width [eax+32], src_height [eax+36], send_event [eax+40])
;;;*  Out: eax=0 (on success); =-1 (on error)
;;;*
;;;*  Write shared memory to window.
;;;*
%macro XShmPutImage 1-11
	extern _XShmPutImage
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 11
	__push %11, %10, %9, %8, %7, %6, %5, %4, %3, %2, %1
	mov eax,esp
%else
 %error "Macro `XShmPutImage' expects 1 or 11 parameters."
%endif
	call _XShmPutImage
%if %0 = 11
	add esp,byte 44
%endif
%endm


%endif	; _x_func_h_


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: