;;;
;;; zip.h
;;;
;;; created: <Sun May 23 16:23:24 1999>
;;; last modified: <Sat Apr  8 13:39:51 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This defines the macros for calling the zip functions in libASM.a
;;;

%ifndef _zip_h_
%define _zip_h_

;;;
;;; ZipGZIP: Gzips n bytes in buf_in to buf_out
;;; In: ZipGZIP *buf_in, n (*buf_in [eax+0], n [eax+4])
;;; Out: eax=buf_out
;;;
;;; buf_out:
;;; 0 dword new size
;;; 4 [dword 0] bytes: data
;;;
%macro ZipGZIP 2
	extern _ZipGZIP
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _ZipGZIP
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;
;;; UnzipGZIP: Gunzips n bytes in buf_in to buf_out
;;; In: UnzipGZIP *buf_in, n (*buf_in [eax+0], n [eax+4])
;;; Out: eax=buf_out
;;;
%macro UnzipGZIP 2
	extern _UnzipGZIP
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _UnzipGZIP
%if %0 = 2
	add esp,byte 8
%endif
%endm


;;;
;;; ZipBZIP2: Bzips n bytes in buf_in to buf_out
;;; In: ZipBZIP2 *buf_in, n (*buf_in [eax+0], n [eax+4])
;;; Out: eax=buf_out
;;;
;;; buf_out:
;;; 0 dword new size
;;; 4 [dword 0] bytes: data
;;;
%macro ZipBZIP2 2
	extern _ZipBZIP2
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _ZipBZIP2
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;
;;; UnzipBZIP2: Bunzips n bytes in buf_in to buf_out
;;; In: UnzipBZIP2 *buf_in, n (*buf_in [eax+0], n [eax+4])
;;; Out: eax=buf_out
;;;
%macro UnzipBZIP2 2
	extern _UnzipBZIP2
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _UnzipBZIP2
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;
;;; ZipLZW: LZW zip n bytes in buf_in to buf_out
;;; In: ZipLZW *buf_in, n (*buf_in [eax+0], n [eax+4])
;;; Out: eax=buf_out (on success); -1 (on error)
;;;
;;; buf_out:	1 dword orig. size
;;;		1 dword compr. size (n)
;;;		1 dword crc32 (uncompressed data)
;;;		n bytes compr. data
;;;
%macro ZipLZW 2
	extern _ZipLZW
%if %0 = 1
 %ifnidni %1,eax
	mov eax,%1
 %endif
%elif %0 = 2
	__push %2, %1
	mov eax,esp
%endif
	call _ZipLZW
%if %0 = 2
	add esp,byte 8
%endif
%endm

;;;
;;; UnzipLZW: LZW zip buf_in to buf_out
;;; In: UnzipLZW *buf_in (eax)
;;; Out: eax=buf_out (on success); -1 (on error)
;;;
;;; buf_in:	1 dword orig. size
;;;		1 dword compr. size (n)
;;;		1 dword crc32 (uncompressed data)
;;;		n bytes compr. data
;;;
%macro UnzipLZW 1
	extern _UnzipLZW
%ifnidni %1,eax
	mov eax,%1
%endif
	call _UnzipLZW
%endm


%endif	; _zip_h_



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: