;;;
;;; WaitKeypressed.s
;;;
;;; created: <Fri Mar 10 11:42:08 2000>
;;; last modified: <Mon Mar 13 19:09:30 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Wait for key and read char from STDIN
;;; In: WaitKeypressed
;;; Out: eax=Char(on success); eax=-1(on error)
;;;

%include "system.h"
%include "io.h"
%include "linux/structs.h"

global _WaitKeypressed

extern _errno

%define key ebp-4
%define old_settings ebp-4-termios_size
%define new_settings ebp-4-termios_size*2

	section .text
	align 32
_WaitKeypressed:
	push ebp
	mov ebp,esp
	sub esp,4+termios_size*2

	lea eax,[old_settings]
	sys_ioctl dword STDIN, dword TCGETS, eax
	cmp eax,-1
	jl near .err

	__push ecx, esi, edi
	lea esi,[old_settings]
	lea edi,[new_settings]
	mov ecx,termios_size
	cld
	rep movsb
	__pop edi, esi, ecx

	;; turn off canonical and echo mode
	;; Unix-Programming-FAQ v1.34, Topics 3.1+3.2
	and [new_settings+termios.c_lflag],dword ~(ICANON|ECHO)
	mov [new_settings+termios.c_cc+VTIME],byte 0
	mov [new_settings+termios.c_cc+VMIN],byte 1

	lea eax,[new_settings]
	sys_ioctl dword STDIN, dword TCSETS, eax

	GetChr
	cmp eax,-1
	je .end
	mov [key],eax

	lea eax,[old_settings]
	sys_ioctl dword STDIN, dword TCSETS, eax
	cmp eax,-1
	jl .err

	mov eax,[key]
.end:
	leave
	ret
.err:
	mov [_errno],eax
	mov eax,-1
	jmp short .end



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: