;;;
;;; WriteFloatScE.s
;;;
;;; created: <Wed Feb 23 10:38:23 2000>
;;; last modified: <Sun Apr 23 21:28:34 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Write Float (qword) *reg to fd with precision prec in scientific format
;;; (xEy = x*10^y)
;;; In: WriteFloatScE fd, *reg, prec (fd [eax+0], *reg [eax+4], prec [eax+8])
;;; Out: -
;;;

;;; Remember: loga(u) = loga(b) * logb(u)      :: log10(x) = log10(2) * log2(x)
;;; and: x^r = (a^loga(x))^r = a^(r * loga(x)) :: 10^x = 2^(x*log2(10))

%include "system.h"
%include "string.h"
%include "io.h"
%include "maths.h"

global _WriteFloatScE
extern _IOE,_IOminus

%define prec ebx+8
%define reg ebx+4
%define fd ebx+0
%define cwbackup ebp-4
%define var ebp-8
%define var2 ebp-16
%define flag ebp-20

	section .text
	align 32
_WriteFloatScE:
	push ebp
	mov ebp,esp
	sub esp,byte 20

	__push eax, ebx
	mov ebx, eax

;;; Save Controlword
	fstcw [cwbackup]

;;; w +/-?
;;; -:	set flag, x=neg w
	mov [flag],dword 0
	mov eax,[reg]
	fld qword [eax]
	fxam
	fstsw ax
	and ah,10b
	jz .positive
	mov [flag],dword 1
	fchs
.positive:
	fld st0

;;; y=rount(log10(x))
	m_log10x
	frndint
	fist dword [var]

;;; z=x/(10^y)
	m_exp10
	fdivp st1

;;; if flag, neg z
	cmp [flag],dword 1
	jne .pos
	fchs
.pos:

;;; WriteFloat (prec) z
	fstp qword [var2]
	lea eax,[var2]
	WriteFloat dword [fd], eax, dword [prec]

;;; Write e
	sys_write dword [fd], dword _IOE, dword 1

;;; WriteInt y
	mov eax,dword [var]
	cmp eax,0
	jnl .wrty
	neg eax
	push eax
 	sys_write dword [fd], dword _IOminus, dword 1
	pop eax
.wrty:
	WriteInt dword [fd], eax

.end:
	fldcw [cwbackup]	; Restore control word

	__pop ebx, eax

	leave
	ret


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: