;;;
;;; PipeIO.s
;;;
;;; created: <Wed May 12 23:41:13 1999>
;;; last modified: <Thu Jun  1 17:06:48 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;


;;;
;;; Example 1:	_PipeIO *fd, *filename, *argv, *envp
;;;		sys_fork
;;;		child:	sys_close dword [fd+0]
;;;			sys_write dword [fd+4], dword *buf, dword n
;;;			sys_close dword [fd+4]
;;;			sys_exit
;;;		sys_close dword [fd+4]
;;;		sys_read dword [fd+0], dword *buf2, dword n2
;;;		sys_close dword [fd+0]
;;;
;;; Example 2:	_PipeIO *fd, *filename, *argv, *envp
;;;		sys_write dword [fd+4], dword *buf, dword n
;;;		sys_close dword [fd+4]
;;;		sys_read dword [fd+0], dword *buf2, dword n2
;;;		sys_close dword [fd+0]
;;;

%include "system.h"
%include "io.h"
%include "sys.h"
%include "string.h"

global _PipeIO
extern _errno,_argv

%define fildes ecx+0
%define filename ecx+4
%define argv ecx+8
%define envp ecx+12
%define fda ecx+16			; Pipe for 4:writing(child), 0:reading(parent)
%define fdb ecx+24			; Pipe for 0:reading(child), 4:writing(parent)

	section .text
	align 32
_PipeIO:
	push ecx
	mov ecx,eax

	;; Create pipe a fd's
	lea eax,[fda]
	sys_pipe eax
	cmp eax,0
	jne near .err

	;; Create pipe b fd's
	lea eax,[fdb]
	sys_pipe eax
	cmp eax,0
	jne near .err

	;; Fork a child
	sys_fork
	cmp eax,0
	jl near .err
	jne near .parent

	;; This is for the child
 	sys_dup2 dword [fdb+0], dword STDIN
	cmp eax,0
	jl near .err
 	sys_dup2 dword [fda+4], dword STDOUT
	cmp eax,0
	jl near .err

	sys_close dword [fda+0]
	sys_close dword [fdb+4]
	mov eax,ecx
	add eax,byte 4
	ExecSP eax
	jmp .err

	;; This is for the parent
.parent:
 	push ebx
 	sys_close dword [fda+4]
	sys_close dword [fdb+0]
 	mov eax,[fildes]
 	mov ebx,[fda+0]
 	mov [eax+0],ebx
 	mov ebx,[fdb+4]
 	mov [eax+4],ebx
 	__pop ecx, ebx
	ret

.err:
	mov [_errno],eax
	GetErrorStr dword [_errno]
	mov ebx,dword [_argv]
	WritefSTDERR '%e: _PipeIO: Could not pipe through %e. (%e)',lf, dword [ebx], dword [filename], eax
	sys_exit dword 1



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: