;;;
;;; MemoryInit.s
;;;
;;; created: <19.10.1998>
;;; last modified: <Thu Jun  1 17:08:47 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; Sets up a memory managment system which uses chunks. If the user application
;;; destroys them, this system will fail and do sys_exit(1).

;;; If the application wants to free allocated memory which is mapped, it should
;;; unmap it first!

;;; The code is self-explaining :-)

%include "system.h"
%include "mem.h"
%include "io.h"

global _MemoryInit,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
global  _MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
global _MemoryChunkStructureIsCorrupted
extern _errno,_argv

	section .text
	align 32

;;;
;;; MemoryInit: Initializes the memory routines.
;;;
;;;  In: MemoryInit
;;;  Out: eax=-1 (on error); 0 (on success)
;;;
;;; Get memory for first Chunk
;;;  [_memory_firstchunk]=([_memory_brk]+PAGE_SIZE-1)&PAGE_MASK
;;;   [_memory_brk] = [_memory_firstchunk]+PAGE_SIZE-1
;;;  Write Chunkheader
;;;  Set mprotect
;;;
_MemoryInit:
	push ebx
	cmp [_memory_HAS_INIT],byte true
	je near .mprotect_OK
; Get adress for the first chunk
	sys_brk dword 0
	mov [_memory_brk],eax
	add eax,PAGE_SIZE-1
	and eax,PAGE_MASK
	mov [_memory_firstchunk],eax

; Get brk
	add eax,PAGE_SIZE
	sys_brk eax
	cmp eax,0
	jge .brk_OK
; Error...
	mov [_errno],eax
	mov eax,-1
	pop ebx
	ret

.brk_OK:
	mov [_memory_brk],eax

; Setup chunk
	mov ebx,[_memory_firstchunk]
	mov [ebx+_memory_chunk.prev],dword 0
	mov [ebx+_memory_chunk.size],dword PAGE_SIZE
	mov [ebx+_memory_chunk.flags],dword _memory_firstchunk_flag|_memory_lastchunk_flag|_memory_identify_flag|_memory_PROT_NONE
	mov [ebx+_memory_chunk.proc],dword 0
; Get pid
	sys_getpid
	mov [ebx+_memory_chunk.pid],eax

; Set protection
	sys_mprotect dword [_memory_firstchunk], dword PAGE_SIZE, dword PROT_NONE
	cmp eax,0
	jnl .mprotect_OK
; Error
	mov [_errno],eax
	mov eax,-1
	pop ebx
	ret

.mprotect_OK:
	mov [_memory_HAS_INIT],byte true
	mov eax,0
	pop ebx
	ret


;;;
;;; This allows READ/WRITE access for all chunkheaders
;;;
	align 32
_MemoryAllowReadWriteHeaders:
	__push eax, esi
	mov esi,[dword _memory_firstchunk]
.Loop:
	sys_mprotect esi, dword PAGE_SIZE, dword PROT_READ|PROT_WRITE
	mov eax,[esi+_memory_chunk.flags]
	and eax,_memory_lastchunk_flag
	jnz .Ende
	add esi,[esi+_memory_chunk.size]
	jmp .Loop
.Ende:
	__pop esi, eax
	ret

;;;
;;; This restores original access for all chunkheaders
;;;
	align 32
_MemoryRestoreHeaders:
	__push ebx, ecx, edx, esi
	mov esi,[dword _memory_firstchunk]
.Loop:
	mov edx,[esi+_memory_chunk.flags]
	mov ebx,edx
	mov ecx,[esi+_memory_chunk.size]
	and ebx,_memory_flag_MPROTmask
	shr ebx,_memory_PROT_shift

	sys_mprotect esi, dword PAGE_SIZE, ebx

	add esi,ecx
	and edx,_memory_lastchunk_flag
	jz .Loop

	__pop esi, edx, ecx, ebx
	ret

;;;
;;; Print out error-string and exit.
;;; eax=Pointer to ASCIIZ-name of calling routine.
;;;
_MemoryChunkStructureIsCorrupted:
	mov ebx,dword [_argv]
	WritefSTDERR '%e: %e: Chunkstructure is corrupted.',lf, dword [ebx], eax
	sys_exit dword 1

	section .bss
	alignb 32
_memory_brk:		resd 1
_memory_firstchunk:	resd 1	; Adress of first Chunk

	section .data
	align 32
_memory_LOCK:		db false	; To LOCK the functions
_memory_HAS_INIT:	db false



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: