;;;
;;; MemoryPrintChunkStruc.s
;;;
;;; created: <19.10.1998>
;;; last modified: <Thu Jun  1 17:08:33 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; Sets up a memory managment system which uses chunks. If the user
;;; application destroys them, this system will fail and do sys_exit(1).

;;; If the application wants to free allocated memory which is mapped, it
;;; should unmap it first!

;;; The code is self-explaining :-)

%include "system.h"
%include "mem.h"
%include "io.h"
%include "string.h"

global _MemoryPrintChunkStruc

extern _errno,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
extern _MemoryInit,_MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
extern _MemoryTune

	section .text
	align 32

;;;
;;; MemoryPrintChunkStruc
;;;
;;;  In: MemoryPrintChunkStruc
;;;  Out: -
;;;
_MemoryPrintChunkStruc:
	pushad
	MemoryHAS_INIT
	MemoryLOCK

	call _MemoryAllowReadWriteHeaders

; Tabellenberschrift
	WritefSTDOUT 'Chunk    Adress   Size     Prev     Flags    Proc     PID',lf
	WritefSTDOUT '------------------------------------------------------------',lf
	mov [_memory_PrintChunkCount],dword 0
	mov [_memory_PrintChunkSum],dword 0
	mov esi,[_memory_firstchunk]

.MainLoop:

	WritefSTDOUT '%d %d %d %d %d %d %a',lf, dword [_memory_PrintChunkCount], esi, dword [esi+_memory_chunk.size], dword [esi+_memory_chunk.prev], dword [esi+_memory_chunk.flags], dword [esi+_memory_chunk.proc], dword [esi+_memory_chunk.pid]
	
	mov eax,[esi+_memory_chunk.size]
	add [_memory_PrintChunkSum],eax

	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz .MainLoopEnd

	inc dword [_memory_PrintChunkCount]
	add esi,[esi+_memory_chunk.size]
	jmp .MainLoop

.MainLoopEnd:
	inc dword [_memory_PrintChunkCount]
	WritefSTDOUT '------------------------------------------------------------',lf
	WritefSTDOUT '%a header(s) with %a bytes.',lf,'Headersize: %a bytes.',lf,  dword [_memory_PrintChunkCount], dword [_memory_PrintChunkSum], dword _memory_chunk_size

	call _MemoryRestoreHeaders

	MemoryUNLOCK
	popad
	ret

	section .bss
	alignb 32
_memory_PrintChunkCount:	resd 1
_memory_PrintChunkSum:	resd 1



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: