;;;
;;; MemoryProtect.s
;;;
;;; created: <19.10.1998>
;;; last modified: <Wed Apr 26 17:01:35 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; Sets up a memory managment system which uses chunks. If the user
;;; application destroys them, this system will fail and do sys_exit(1).

;;; If the application wants to free allocated memory which is mapped, it
;;; should unmap it first!

;;; The code is self-explaining :-)

%include "system.h"
%include "mem.h"
%include "io.h"

global _MemoryProtect

extern _errno,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
extern _MemoryInit,_MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
extern _MemoryTune,_MemoryChunkStructureIsCorrupted

%define flags ebp+4
%define ptr ebp+0

	section .text
	align 32

;;;
;;; MemoryProtect
;;;
;;;  In: MemoryProtect ptr, flags (ptr [eax+0], flags [eax+4])
;;;  Out: eax=0 (success), -1 (error)
;;;
;;; NOTE: This changes the protection for the hole chunk. If You want to change
;;; only a part of it, you can do this via sys_mprotect. Maybe that the lib code
;;; will fail or will change the protection. You have been warned.
;;;
_MemoryProtect:
	__push edx, esi, edi, ebp
	mov ebp,eax

	MemoryHAS_INIT
	MemoryLOCK

	call _MemoryAllowReadWriteHeaders

; Verify Argument
	mov esi,[ptr]
	sub esi,_memory_chunk_size	; need header adress
	mov edi,[_memory_firstchunk]
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz .NotValid

.VLoop:
; Check if ID is OK, if not, exit an give ERROR!
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_flag_IDmask
	cmp edx,_memory_identify_flag
	je .VLoop_j1
	mov eax,dword .rout_name
	jmp _MemoryChunkStructureIsCorrupted
.VLoop_j1:

	add edi,[edi+_memory_chunk.size]
	cmp edi,esi
	je .TestFlagOK

	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .VLoop

; Sorry, this is not a valid Pointer
.NotValid:
	mov [_errno],dword -EINVAL
.NotValid2:
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop ebp, edi, esi, edx
	mov eax,-1
	ret

.TestFlagOK:
;;; Now check if chunk access is forbidden
	mov eax,[esi+_memory_chunk.flags]
	and eax,_memory_forbidmodify_flag
	jz .TestFlagOK2
;;; Is this the same job?
	sys_getpid
	cmp [esi+_memory_chunk.pid],eax
	je .TestFlagOK2
	mov [_errno],dword -EFAULT
	jmp short .NotValid2

.TestFlagOK2:
	mov eax,[flags]
	and eax,(_memory_flag_MPROTmask|_memory_forbidmodify_flag)

; OK. Set the .flags field and notify the kernel about the change.
	and [esi+_memory_chunk.flags],dword ~(_memory_flag_MPROTmask|_memory_forbidmodify_flag)
	or [esi+_memory_chunk.flags],eax

	and eax,_memory_flag_MPROTmask
	shr eax,_memory_PROT_shift
	sys_mprotect esi, dword [esi+_memory_chunk.size],eax
	sys_mprotect esi, dword PAGE_SIZE, dword  PROT_READ|PROT_WRITE

	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop ebp, edi, esi, edx
	mov eax,0
	ret

	section .data
.rout_name:	db '_MemoryProtect',0



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: