;;;
;;; MemoryResize.s
;;;
;;; created: <19.10.1998>
;;; last modified: <Wed Apr 26 17:02:26 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; Sets up a memory managment system which uses chunks. If the user
;;; application destroys them, this system will fail and do sys_exit(1).

;;; If the application wants to free allocated memory which is mapped, it
;;; should unmap it first!

;;; The code is self-explaining :-)

%include "system.h"
%include "mem.h"
%include "io.h"
%include "string.h"

global _MemoryResize

extern _errno,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
extern _MemoryInit,_MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
extern _MemoryTune,_MemoryChunkStructureIsCorrupted

%define newsize ebp+4
%define ptr ebp+0

	section .text
	align 32

;;;
;;; MemoryResize
;;;
;;;  In: MemoryResize ptr, newsize (ptr [eax+0], newsize [eax+4])
;;;  Out: eax=(new) ptr (on success); -1 (on error)
;;;
_MemoryResize:
	__push ebx, ecx, edx, esi, edi, ebp
	mov ebp,eax

	MemoryHAS_INIT
	MemoryLOCK

	call _MemoryAllowReadWriteHeaders

; Verify Argument
	mov esi,[ptr]
	sub esi,byte _memory_chunk_size	; need header adress
	mov edi,[_memory_firstchunk]
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz .NotValid

.VLoop:
; Check if ID is OK, if not, exit an give ERROR!
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_flag_IDmask
	cmp edx,_memory_identify_flag
	je .VLoop_j1
	mov eax,dword .rout_name
	jmp _MemoryChunkStructureIsCorrupted
.VLoop_j1:

	add edi,[edi+_memory_chunk.size]
	cmp edi,esi
	je .TestFlagOK

	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .VLoop

; Sorry, this is not a valid Pointer
.NotValid:
	mov [_errno],dword -EINVAL
.NotValid2:
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop ebp, edi, esi, edx, ecx, ebx
	mov eax,-1
	ret

.TestFlagOK:
;;; Now check if chunk access is forbidden
	mov eax,[esi+_memory_chunk.flags]
	and eax,_memory_forbidmodify_flag
	jz .TestFlagOK2
;;; Is this the same job?
	sys_getpid
	cmp [esi+_memory_chunk.pid],eax
	je .TestFlagOK2
	mov [_errno],dword -EFAULT
	jmp short .NotValid2

.TestFlagOK2:
; Size in Pages
	mov eax,[newsize]
	add eax,_memory_chunk_size	; Usermemory is chunk.size-chunk_size
	add eax,PAGE_SIZE-1
	and eax,PAGE_MASK

; If old and newsize are equal, exit
	cmp eax,[edi+_memory_chunk.size]
	je near .OK_End

; If the new is smaller
	ja near .NewBigger

; Do this
	mov ebx,[edi+_memory_chunk.size]
	sub ebx,eax
	mov ecx,[edi+_memory_chunk.size]

; Modify header
	mov [edi+_memory_chunk.size],eax

; Create next
	mov esi,edi
	add esi,eax
; Mprotect
	push eax
	sys_mprotect esi, ebx, dword  PROT_READ|PROT_WRITE
	pop eax

	mov [esi+_memory_chunk.prev],eax
	mov [esi+_memory_chunk.size],ebx

; If the modified Chunk was the last, the new one is the last
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .NewSmallerNotLast

	mov [esi+_memory_chunk.flags],dword _memory_lastchunk_flag|_memory_identify_flag|_memory_PROT_NONE|_memory_freechunk_flag
	and [edi+_memory_chunk.flags],dword ~(_memory_lastchunk_flag)
	jmp short .NewSmallerGoon

.NewSmallerNotLast:
	mov [esi+_memory_chunk.flags],dword _memory_identify_flag|_memory_PROT_NONE|_memory_freechunk_flag
	mov [esi+ebx+_memory_chunk.prev],ebx

.NewSmallerGoon:
	mov [esi+_memory_chunk.proc],dword 0
	sys_mprotect esi, dword [esi+_memory_chunk.size], dword  PROT_NONE
	sys_mprotect esi, dword PAGE_SIZE, dword  PROT_READ|PROT_WRITE

	jmp .OK_End


; The new Chunk is bigger than the old
.NewBigger:
; Is it the last? Add new BRK
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz near .NewBiggerNotLast

	mov ebx,eax
	add eax,edi
	sys_brk eax
; Error?
	cmp eax,0
	jnle .AllocNewBrkOK

; Return Error
	mov [_errno],eax
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop ebp, edi, esi, edx, ecx, ebx
	mov eax,-1
	ret

.AllocNewBrkOK:
	mov [_memory_brk],eax
	mov [edi+_memory_chunk.size],ebx

	jmp .OK_End

.NewBiggerNotLast:
; Is the next one free
	mov edx,[edi+_memory_chunk.size]
	add edx,[edi+edx+_memory_chunk.flags]
	and edx,_memory_freechunk_flag
	jnz near .NewBiggerNextFree

;;; Get new chunk, because the used one is too small!
.getNewChunk:
;;; Restore Headers & Unlock
	call _MemoryRestoreHeaders
	MemoryUNLOCK

;;; Allocate new chunk (newsize)
	MemoryAllocate dword [newsize]
	cmp eax,-1
	je near .ENOMEM_j1
	push eax

;;; Move memoryblock
	mov eax,dword [ptr]
	sub eax,byte _memory_chunk_size
	sys_mprotect eax, dword [edi+_memory_chunk.size], dword  PROT_READ|PROT_WRITE
	pop eax
	push eax
	mov ebx,dword [edi+_memory_chunk.size]
	sub ebx,byte _memory_chunk_size
	Strncpy dword [ptr], eax, ebx

;;; Set Protection
	mov ebx,[edi+_memory_chunk.flags]
	and ebx,_memory_flag_MPROTmask|_memory_forbidmodify_flag
	pop eax
	push eax
;	sub eax,byte _memory_chunk_size
	MemoryProtect eax, ebx

;;; Get .proc
	pop ebx
	push ebx
	sub ebx,byte _memory_chunk_size
	mov eax,[edi+_memory_chunk.proc]
	mov [ebx+_memory_chunk.proc],eax
;;; Set old one to NULL
	mov [edi+_memory_chunk.proc],dword 0

;;; Free old chunk
	MemoryFree dword [ptr]

;;; Return new pointer
	pop eax
	__pop ebp, edi, esi, edx, ecx, ebx
	ret




; Sorry, can't change size: ENOMEM
.ENOMEM:
	mov [_errno],dword -ENOMEM
	call _MemoryRestoreHeaders
	MemoryUNLOCK
.ENOMEM_j1:
	__pop ebp, edi, esi, edx, ecx, ebx
	mov eax,-1
	ret




.NewBiggerNextFree:
; The next Chunk seems to be free. Is there enough space left?
	mov edx,[edi+_memory_chunk.size]
	add edx,[edi+edx+_memory_chunk.size]
	cmp eax,edx
	;; 	ja .ENOMEM
	ja near .getNewChunk

.NewBiggerNextSizeOK:
; We don't need to check if this one +1 could be the last Chunk. If tune is ok,
; this will never be the last at this moment.
	mov ebx,[edi+_memory_chunk.size]
	mov esi,edi
	add esi,ebx
	mov ecx,[esi+_memory_chunk.size]

; We need to free the hole free chunk
	push eax
	sys_mprotect esi, ecx, dword  PROT_READ|PROT_WRITE
	pop eax

; Modify header
	mov [edi+_memory_chunk.size],eax

; Create next header
	mov edx,eax
	sub edx,ebx
	sub ecx,edx	; new size of free chunk

	mov esi,edi
	add esi,eax

	mov [esi+_memory_chunk.flags],dword _memory_identify_flag|_memory_PROT_NONE|_memory_freechunk_flag
	mov [esi+_memory_chunk.size],ecx
	mov [esi+_memory_chunk.prev],eax

; Protect the free chunk
	sys_mprotect esi, ecx, dword PROT_NONE
	sys_mprotect esi, dword PAGE_SIZE, dword PROT_READ|PROT_WRITE

; Modify the .prev of the header following the newone
	mov [esi+ecx+_memory_chunk.prev],ecx

.OK_End:

	call _MemoryRestoreHeaders
	MemoryUNLOCK
	mov eax,dword [ptr]
	__pop ebp, edi, esi, edx, ecx, ebx
; Tune Memory!
	call _MemoryTune
	;; mov eax,0
	ret

	section .data
.rout_name:	db '_MemoryResize',0



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: