;;;
;;; MemorySetProc.s
;;;
;;; created: <Sun Apr 16 19:19:55 2000>
;;; last modified: <Wed Apr 26 17:09:39 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; MemorySetProc
;;;
;;;  In: MemorySetProc ptr, proc, (ptr [eax+0], proc [eax+4])
;;;  Out: 0 (success), -1 (error)
;;;
;;; This sets the pointer to a procedure that will be called before the chunk
;;; will be released. arg is the eax value for the procedure. If proc is NULL
;;; nothing will be called.

%include "system.h"
%include "mem.h"
%include "io.h"

global _MemorySetProc

extern _errno,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
extern _MemoryInit,_MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
extern _MemoryTune,_MemoryChunkStructureIsCorrupted

	section .text
	align 32

_MemorySetProc:
	__push ebx, edx, esi, edi

	mov esi,[eax+0]
	mov ebx,[eax+4]

	MemoryHAS_INIT		; '_MemorySetProc: '
	MemoryLOCK

	call _MemoryAllowReadWriteHeaders

; Verify Argument
	sub esi,_memory_chunk_size	; need header adress
	mov edi,[_memory_firstchunk]
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz .NotValid

.VLoop:
; Check if ID is OK, if not, exit an give ERROR!
	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_flag_IDmask
	cmp edx,_memory_identify_flag
	je .VLoop_j1
	mov eax,dword .rout_name
	jmp _MemoryChunkStructureIsCorrupted
.VLoop_j1:

	add edi,[edi+_memory_chunk.size]
	cmp edi,esi
	je .TestFlagOK

	mov edx,[edi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .VLoop

; Sorry, this is not a valid Pointer
.NotValid:
	mov [_errno],dword -EINVAL
.NotValid2:
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop edi, esi, edx, ebx
	mov eax,-1
	ret

.TestFlagOK:
;;; Now check if chunk access is forbidden
	mov eax,[esi+_memory_chunk.flags]
	and eax,_memory_forbidmodify_flag
	jz .TestFlagOK2
;;; Is this the same job?
	sys_getpid
	cmp [esi+_memory_chunk.pid],eax
	je .TestFlagOK2
	mov [_errno],dword -EFAULT
	jmp short .NotValid2

.TestFlagOK2:
	mov [esi+_memory_chunk.proc],ebx

	call _MemoryRestoreHeaders
	MemoryUNLOCK
	__pop edi, esi, edx, ebx
; Tune Memory!
	call _MemoryTune
	mov eax,0
	ret

	section .data
.rout_name:	db '_MemorySetProc',0



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: