;;;
;;; MemoryTune.s
;;;
;;; created: <19.10.1998>
;;; last modified: <Wed Apr 26 17:04:01 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;
;;; This is to handle memory allocation in assembly much easier.
;;;

;;; Sets up a memory managment system which uses chunks. If the user
;;; application destroys them, this system will fail and do sys_exit(1).

;;; If the application wants to free allocated memory which is mapped, it
;;; should unmap it first!

;;; The code is self-explaining :-)

%include "system.h"
%include "mem.h"
%include "io.h"

global _MemoryTune

extern _errno,_memory_brk,_memory_firstchunk,_memory_LOCK,_memory_HAS_INIT
extern _MemoryInit,_MemoryAllowReadWriteHeaders,_MemoryRestoreHeaders
extern _MemoryChunkStructureIsCorrupted

	section .text
	align 32

;;;
;;; MemoryTune
;;;
;;;  In: MemoryTune
;;;  Out: -
;;;
;;;Header freigeben
;;;1:
;;; Freien Chunk suchen (=a)
;;;  a.flags=last-> 2:
;;;  rek:
;;;   nchster frei? (=b)
;;;    nein-> end.rek
;;;    ja:
;;;     b.flags=last: 
;;;      a.flags:=last
;;;      a.size:=a.size+b.size
;;;      -> end.rek
;;;     a.size:=a.size+b.size
;;;  -> rek:
;;;  end.rek:
;;; a.flags=last-> 2:
;;; Nchster Chunk: (a+a.size).prev:=a.size
;;; sys_mprot=a,a.size,none
;;; sys_mprot=a,page,read|write
;;; -> 1:
;;;2:
;;; Letzten Chunk suchen (=a)
;;;  a.flags=frei
;;;   (a-a.prev).flags:=last
;;;   sys_brk
;;;Header zurck
;;;ret
_MemoryTune:
	pushad

	MemoryHAS_INIT
	MemoryLOCK

	call _MemoryAllowReadWriteHeaders

	mov esi,[_memory_firstchunk]
.SearchLoop:
; Check if ID is OK, if not, exit an give ERROR!
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_flag_IDmask
	cmp edx,_memory_identify_flag
	je .SearchLoop_j1
	mov eax,dword .rout_name
	jmp _MemoryChunkStructureIsCorrupted
.SearchLoop_j1:
; Check if Chunk is free
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_freechunk_flag
	jnz .IsFreeChunk
	add esi,[esi+_memory_chunk.size]
; Last chunk?
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .SearchLoop
; Operation finished.
	jmp .PartTwo
.IsFreeChunk:
	mov edi,esi	; This is the free Chunk
.Rekursive:
	add esi,[esi+_memory_chunk.size]
; Check if Chunk is free
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_freechunk_flag
	jz .RekursiveEnd
; If this is the last, mark the Chunk in edi
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz .Rekursive_j1
	and [edi+_memory_chunk.flags],dword ~(_memory_lastchunk_flag)
	or [edi+_memory_chunk.flags],dword _memory_lastchunk_flag
	mov eax,[esi+_memory_chunk.size]
	add [edi+_memory_chunk.size],eax
	jmp .RekursiveEnd
.Rekursive_j1:
	mov eax,[esi+_memory_chunk.size]
	add [edi+_memory_chunk.size],eax
	jmp .Rekursive
.RekursiveEnd:
	mov esi,edi
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz near .PartTwo
; Modify the .prev of the next chunk (new a.size)
	mov eax,[esi+_memory_chunk.size]
	mov [esi+eax+_memory_chunk.prev],eax
; Protect the free Memory
	push eax
	sys_mprotect esi, eax, dword PROT_NONE
	sys_mprotect esi, dword PAGE_SIZE, dword PROT_READ|PROT_WRITE
	pop eax
	add esi,eax
; Last chunk?
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jz near .SearchLoop

.PartTwo:
	mov esi,[_memory_firstchunk]
.P2SearchLoop:
; Find last
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_lastchunk_flag
	jnz .P2FoundLast
	add esi,[esi+_memory_chunk.size]
	jmp .P2SearchLoop

.P2FoundLast:
; Found last. is Free
	mov edx,[esi+_memory_chunk.flags]
	and edx,_memory_freechunk_flag
	jz .End

	mov eax,[esi+_memory_chunk.prev]
	neg eax
	and [esi+eax+_memory_chunk.flags],dword ~(_memory_lastchunk_flag)
	or [esi+eax+_memory_chunk.flags],dword _memory_lastchunk_flag

	mov [_memory_brk],esi
	sys_brk esi

.End:
	call _MemoryRestoreHeaders
	MemoryUNLOCK
	popad
	ret

	section .data
.rout_name:	db '_MemoryTune',0



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: