;;;
;;; SEQ_open.s
;;;
;;; created: <Sun Jun  6 15:27:00 1999>
;;; last modified: <Thu Jun  1 17:09:14 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

global _SEQ_open
extern _errno,_argv

%include "system.h"
%include "linux/ioctl.h"
%include "seq.h"
%include "linux/oss.h"
%include "io.h"
%include "string.h"
%include "mem.h"

%define _seqdev esi+0
%define Type esi+4
%define Subtype esi+8

%define _snd ebp-4
%define SEQ_device_numbers ebp-8
%define SEQ_device_info ebp-8-synth_info_size

	section .text
	align 32
_SEQ_open:
	push ebp
	mov ebp,esp
	sub esp,8+synth_info_size
	__push ebx, ecx, edx, esi
	mov esi,eax

	;; Allocate memory for snd struc
	MemoryAllocate dword snd_size
	cmp eax,0
	jl near .exit		; _errno and eax set by MemoryAllocate
	mov [_snd],eax
	mov ebx,eax

	lea eax,[ebx+snd.seqdev_name] ; WARNING: string should not be longer than 256 byte!
	Strcpy dword [_seqdev], eax

	;; Open sequencer
	lea eax,[ebx+snd.seqdev_name]
	sys_open eax, dword O_RDWR, dword 0
	cmp eax,0
	jl near .err
	mov [ebx+snd.fd],eax

	;; Get Number of sound devices
	lea eax,[SEQ_device_numbers]
	sys_ioctl dword [ebx+snd.fd], dword SNDCTL_SEQ_NRSYNTHS, eax
	cmp eax,0
	jl near .err

	mov ecx,[SEQ_device_numbers]
	mov edx,0
.finddev:
	mov [SEQ_device_info+synth_info.device],edx
	lea eax,[SEQ_device_info]
	sys_ioctl dword [ebx+snd.fd], dword SNDCTL_SYNTH_INFO, eax
	cmp eax,0
	jl near .err

	mov eax,[Type]
	cmp [SEQ_device_info+synth_info.synth_type],eax
	je .istype
	jmp .isnottype
.istype:
	mov eax,[Subtype]
	cmp [SEQ_device_info+synth_info.synth_subtype],eax
	je .issubtype
.isnottype:
	inc edx
	dec ecx
	jnz .finddev

	;; Show what we've not found:
	mov eax,dword [_argv]
	lea ebx,[ebx+snd.seqdev_name]
	WritefSTDERR '%e: _SEQ_open: Sorry, specified sound device (%a,%a) not present on device %e.',lf, dword [eax], dword [Type], dword [Subtype], ebx
	mov eax,-ENXIO
	jmp .err
	;; sys_exit dword -1

.issubtype:
	cmp [Type],dword SYNTH_TYPE_SAMPLE
	jne near .notsample
	;; Reset samples on SEQ found
	lea eax,[SEQ_device_info+synth_info.device]
	sys_ioctl dword [ebx+snd.fd], dword SNDCTL_SEQ_RESETSAMPLES, eax
	cmp eax,0
	jl near .err
	;; Get onboard mem
	mov eax,[SEQ_device_info+synth_info.device]
	mov [ebx+snd.memavail],eax
	lea eax,[ebx+snd.memavail]
	sys_ioctl dword [ebx+snd.fd], dword SNDCTL_SYNTH_MEMAVL, eax
	cmp eax,0
	jl near .err
.notsample:

;;; Setup snd struc
	mov [ebx+snd.seqbufptr],dword 0
	mov eax,[SEQ_device_info+synth_info.device]
	mov [ebx+snd.device],eax
	mov eax,[SEQ_device_info+synth_info.synth_type]
	mov [ebx+snd.synth_type],eax
	mov eax,[SEQ_device_info+synth_info.synth_subtype]
	mov [ebx+snd.synth_subtype],eax
	mov eax,[SEQ_device_info+synth_info.perc_mode]
	mov [ebx+snd.perc_mode],eax
	mov eax,[SEQ_device_info+synth_info.nr_voices]
	mov [ebx+snd.nr_voices],eax
	mov eax,[SEQ_device_info+synth_info.nr_drums]
	mov [ebx+snd.nr_drums],eax
	mov eax,[SEQ_device_info+synth_info.instr_bank_size]
	mov [ebx+snd.instr_bank_size],eax
	mov eax,[SEQ_device_info+synth_info.capabilities]
	mov [ebx+snd.capabilities],eax
	lea eax,[SEQ_device_info+synth_info.dummies]
	lea edx,[ebx+snd.dummies]
	Strncpy eax, edx, dword 19*4
	lea eax,[SEQ_device_info+synth_info.name]
	lea edx,[ebx+snd.name]
	Strncpy eax, edx, dword 30

%ifdef ___VERBOSE_FLAG___
	;; Show what we've found:
	lea eax,[ebx+snd.seqdev_name]
	lea ecx,[ebx+snd.name]
	mov edx,dword [_argv]
	WritefSTDOUT '%e: _SEQ_open: Sound device %a (%e): %e',lf, dword [edx], dword [SEQ_device_info+synth_info.device], eax, ecx
%endif	; ___VERBOSE_FLAG___

	;; Success
	mov eax,[_snd]

.exit:
	__pop esi, edx, ecx, ebx
	leave
	ret


.err:
	push eax
	MemoryFree dword [_snd]
	pop eax
	mov [_errno],eax
	mov eax,-1
	jmp short .exit



;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: