;;;
;;; Strncpy.s
;;;
;;; created: <Sat Oct  2 23:24:08 1999>
;;; last modified: <Sat Nov 15 22:07:22 2003>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Copy len bytes from src to dest
;;; In: Strncpy src, dest, len (src [eax+0], dest [eax+4], len [eax+8])
;;; Out: eax=(src [eax+0], dest [eax+4], len [eax+8])
;;;

%include "system.h"
%include "misc.h"
%include "io.h"

global _Strncpy,_StrncpySetup
extern _argv

%define len eax+8
%define dest eax+4
%define src eax+0

%define cputype ebp-CPUInfo_size

	section .text
	align 16
_StrncpySetup:
	push ebp
	mov ebp,esp
	sub esp,CPUInfo_size

	push eax
	lea eax,[cputype]
	GetCPUType eax		; Only needed to do the following
;;; .x86Copy is default.
	mov [_Strncpy.jumpto],dword _Strncpy.x86Copy

;;; MMXExt/3DNOWExt
%ifdef ___MMXExt___
%ifdef ___3DNOWExt___
	mov eax,[cputype+CPUInfo.CPUExtensions]
	and eax,dword CPUInfo_HaveMMXExt|CPUInfo_Have3DNowExt
	jz .noMMXExtCopy
	mov [_Strncpy.jumpto],dword _Strncpy.MMXExtCopy
%ifdef ___VERBOSE_FLAG___
	mov eax,dword [_argv]
	WritefSTDOUT '%e: _Strncpy: Using MMXExt/3DNOWExt extensions.',lf, dword [eax]
%endif	; ___VERBOSE_FLAG___
	jmp short .end
	align 16
.noMMXExtCopy:
%endif	; ___3DNOW___
%endif	; ___MMXExt___

;;; MMX?
%ifdef ___MMX___
	and [cputype+CPUInfo.CPUExtensions],dword CPUInfo_HaveMMX
	jz .noMMXCopy
	mov [_Strncpy.jumpto],dword _Strncpy.MMXCopy
%ifdef ___VERBOSE_FLAG___
	mov eax,dword [_argv]
	WritefSTDOUT '%e: _Strncpy: Using MMX extensions.',lf, dword [eax]
%endif	; ___VERBOSE_FLAG___
	jmp short .end
	align 16
.noMMXCopy:
%endif	; ___MMX___
	

;;; 686?
	cmp [cputype+CPUInfo.ProcessorType],dword 3
	jb .no686Copy
	mov [_Strncpy.jumpto],dword _Strncpy.686Copy
	jmp short .end
	align 16
.no686Copy:
.end:
	pop eax
	leave
	jmp dword [_Strncpy.jumpto]

	align 16
_Strncpy:
	jmp dword [.jumpto]

;;; This is optimized for 686
	align 16
.686Copy:
	__push eax, ecx, esi, edi

	mov ecx,[len]
	mov esi,[src]
	mov edi,[dest]
	mov eax,ecx
	cld

	and eax,3
	shr ecx,2
	rep movsd		; rep movs does not move if ecx=0

	mov ecx,eax
	rep movsb		; rep movs does not move if ecx=0

	__pop edi, esi, ecx, eax
	ret


;;; This is optimized for 386, 486, 586
	align 16
.x86Copy:
	__push eax, ebx, ecx, esi, edi
	mov ecx,[len]
	mov esi,[src]
	mov edi,[dest]
	mov eax,ecx
	cld

	and eax,~(31)
	jz .x86j2
	align 16
.x86j1:
	sub eax,byte 32
	mov ebx,[esi+eax+0]
	mov ecx,[esi+eax+4]
	mov [edi+eax+0],ebx
	mov [edi+eax+4],ecx
	mov ebx,[esi+eax+8]
	mov ecx,[esi+eax+12]
	mov [edi+eax+8],ebx
	mov [edi+eax+12],ecx
	mov ebx,[esi+eax+16]
	mov ecx,[esi+eax+20]
	mov [edi+eax+16],ebx
	mov [edi+eax+20],ecx
	mov ebx,[esi+eax+24]
	mov ecx,[esi+eax+28]
	mov [edi+eax+24],ebx
	mov [edi+eax+28],ecx
	jnz .x86j1

.x86j2:
	mov eax,ecx
	and ecx,31		; rep movs does not move if ecx=0
   	and eax,~(31)
	add esi,eax
	add edi,eax
	rep movsb

	__pop edi, esi, ecx, ebx, eax
	ret

%ifdef ___MMX___
;;; Optimized for MMX
	align 16
.MMXCopy:
	__push eax, ebx, ecx, esi, edi
	mov ecx,[len]
	mov ebx,[src]
	mov edi,[dest]
	mov eax,ecx
	cld

	shr eax,4
	jz .MMXj2
	align 16
.MMXj1:
	movq mm0,[ebx]
	movq mm1,[ebx+8]
	movq [edi],mm0
	movq [edi+8],mm1
	add ebx,byte 16
	add edi,byte 16
	dec eax
	jnz .MMXj1
	emms

.MMXj2:
	and ecx,15
	mov esi,ebx
	rep movsb

	__pop edi, esi, ecx, ebx, eax
	ret
%endif	; ___MMX___





	
%ifdef ___MMXExt___
%ifdef ___3DNOWExt___
;;; Optimized for MMXExt/3DNOWExt
;;; 'Inspired' by "AMD Athlon Processor x86 Code Optimization Guide"
	align 16
.MMXExtCopy:
	__push eax, ebx, ecx, esi, edi
	mov ecx,[len]
	mov esi,[src]
	mov edi,[dest]
	cld

	cmp ecx,7		; Smaller than 7 byte? No optimization needed...
	jb near .MMXExt_rest
.MMXExt_aligndest:
	mov eax,8
	sub eax,edi
	and eax,7
	sub ecx,eax		; Ajust length
	neg eax
	add eax,.MMXExt_aligndest_jmp
	jmp eax
	align 16
	movsb
	movsb
	movsb
	movsb
	movsb
	movsb
	movsb
	movsb
.MMXExt_aligndest_jmp:
	;; edi is now aligned
;; jmp .MMXExt_smaller
.MMXExt_main:
	cmp ecx,8192		; Bigger than cache page?
	jb near .MMXExt_smaller
	;; Fill the cache
	mov eax,64		; 8192/64/2
	add esi,8192		; Top of this 8k block
.MMXExt_prefetch:
	mov ebx,[esi-64]	; one cache line
	mov ebx,[esi-128]	; one cache line
	sub esi,128
	dec eax
	jnz .MMXExt_prefetch
	;; Cache is filled, copy the data
	mov eax,128		; 8192/64
.MMXExt_copy8k:
	movq mm0,[esi+0]
	movq mm1,[esi+8]
	movq mm2,[esi+16]
	movq mm3,[esi+24]
	movq mm4,[esi+32]
	movq mm5,[esi+40]
	movq mm6,[esi+48]
	movq mm7,[esi+56]
	movntq [edi+0],mm0	; Write but don't use the cache
	movntq [edi+8],mm1
	movntq [edi+16],mm2
	movntq [edi+24],mm3
	movntq [edi+32],mm4
	movntq [edi+40],mm5
	movntq [edi+48],mm6
	movntq [edi+56],mm7
	add esi,byte 64
	add edi,byte 64
	dec eax
	jnz .MMXExt_copy8k
	sub ecx,8192
	jmp short .MMXExt_main

	align 16
.MMXExt_smaller:		; Copy 64 Byte Blocks
	mov eax,ecx
	shr eax,6
	jz .MMXExt_rest4
.MMXExt_smaller2:
	movq mm0,[esi+0]
	movq mm1,[esi+8]
	movq mm2,[esi+16]
	movq mm3,[esi+24]
	movq mm4,[esi+32]
	movq mm5,[esi+40]
	movq mm6,[esi+48]
	movq mm7,[esi+56]
	movntq [edi+0],mm0	; Write but don't use the cache
	movntq [edi+8],mm1
	movntq [edi+16],mm2
	movntq [edi+24],mm3
	movntq [edi+32],mm4
	movntq [edi+40],mm5
	movntq [edi+48],mm6
	movntq [edi+56],mm7
	add esi,byte 64
	add edi,byte 64
	sub ecx,byte 64
	dec eax
	jnz .MMXExt_smaller2
	
.MMXExt_rest4:
	mov eax,ecx
	shr eax,2
	jz .MMXExt_rest
	and eax,63
	mov ebx,eax
	shl ebx,2
	sub ecx,ebx
	neg eax
	add eax,.MMXExt_rest4_done
	jmp eax
	align 16
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
	movsd
.MMXExt_rest4_done:

.MMXExt_rest:
	and ecx,3
	rep movsb
	femms
	sfence

	__pop edi, esi, ecx, ebx, eax
	ret
%endif	; ___3DNOW___
%endif	; ___MMXExt___


	section .data
.jumpto:	dd _StrncpySetup


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End:
