;;;
;;; ExecSP.s
;;;
;;; created: <Mon May 10 13:42:23 1999>
;;; last modified: <Thu Jun  1 17:06:04 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;; In: ExecSP *filename, **argv, **envp (*filename [eax+0], **argv [eax+4], **envp [eax+8])
;;; Out: never returns


%include "system.h"
%include "io.h"
%include "string.h"

global _ExecSP
extern _errno,_argv,_envp

%define filename ecx+0
%define argv ecx+4
%define envp ecx+8

%define path ebp-512

	section .text
	align 32
_ExecSP:
	push ebp
	mov ebp,esp
	sub esp,512	; 512 Bytes of local data for 'path'

	mov ecx,eax

	;; We will search the PATH env for valid dir only if [filename]
	;; has no '/'
	mov esi,[filename]
.findslash:
	cmp [esi],byte 0
	je .noslash
	cmp [esi],byte '/'
	je .foundslash
	inc esi
	jmp .findslash

.foundslash:
	sys_execve dword [filename], dword [argv], dword [envp]
	mov [_errno],eax
	jmp .err

.noslash:
	;; Find 'PATH' environment
	mov ebx,[_envp]
.findpath:
	mov esi,[ebx]
	cmp [ebx],dword 0
	je near .nopath
	cmp [esi],dword 'PATH'
	jne .notyetfound
	cmp [esi+4],byte '='
	je .foundpath
.notyetfound:
	add ebx,byte 4
	jmp short .findpath

.foundpath:
	;; Scan the PATH env for the valid dir
	add esi,byte 5
.scan:
	cmp [esi],byte 0
	je .nopath
	lea edi,[path]

.scan_j1:
	cmp [esi],byte ':'
	je .test
	movsb	; copy path
	jmp .scan_j1

.test:
	inc esi
	mov al,'/'
	stosb
	push esi
	mov esi,[filename]
.test_j1:
	cmp [esi],byte 0
	je .test_j2
	movsb
	jmp .test_j1
.test_j2:
	movsb
	pop esi

	lea eax,[path]
	sys_execve eax, dword [argv], dword [envp]
	jmp .scan

	;; No path env, try to use the current dir for that
.nopath:
	mov esi,[filename]
	lea edi,[path]
	mov ax,'./'
	stosw
.nopathj1:
	cmp [esi],byte 0
	je .nopathj2
	movsb
	jmp .nopathj1
.nopathj2:
	movsb

	lea eax,[path]
	sys_execve eax, dword [argv], dword [envp]
	mov [_errno],eax

.err:
	GetErrorStr dword [_errno]
	mov ebx,dword [_argv]
	WritefSTDERR '%e: _ExecSP: Could not execute %e. (%e)',lf, dword [ebx], dword [filename], eax
	sys_exit dword 1


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: