;;;
;;; XOpenDisplay.s
;;;
;;; created: <Wed Nov  3 16:17:02 1999>
;;; last modified: <Fri Sep 22 20:36:13 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

%include "system.h"
%include "io.h"
%include "x.h"
%include "mem.h"
%include "sys.h"
%include "string.h"

global _XOpenDisplay
extern _envp,_errno,_argv

%define display ebp-4
%define displaystr_size ebp-8
%define displaystr ebp-12
%define protokoll ebp-16
%define hostname ebp-20
%define display_nr ebp-24
%define screen_nr ebp-28
%define fd ebp-32
%define info ebp-36
%define dpy ebp-40
%define client ebp-40-xConnClientPrefix_size
%define XSetupPrefix ebp-40-xConnClientPrefix_size-xConnSetupPrefix_size

	section .text
	align 32
_XOpenDisplay:
	push ebp
	mov ebp,esp
	sub esp,40+xConnClientPrefix_size+xConnSetupPrefix_size

	__push ebx, ecx, edx, esi, edi
	mov [display],eax

	mov [displaystr],dword 0
	mov [fd],dword 0
	mov [info],dword 0
	mov [dpy],dword 0
;;;
;;; First: Get the displayname. If the pointer is 0 try to get it from the
;;; environment variable 'DISPLAY'
;;;
	cmp [display],dword 0
	jne .HaveDisplay

;;; Get it from the environment...
	mov ebx,[_envp]
	sub ebx,byte 4
.envloop:
	add ebx,byte 4
	cmp [ebx],dword 0
	je near ConnectionError
	mov esi,[ebx]
	mov edi,display_str
	mov ecx,display_strlen
	cld
	repe cmpsb
	cmp ecx,0
	jne .envloop
	mov [display],esi

.HaveDisplay:
	mov eax,[display]
	cmp [eax],byte 0
	je near ConnectionError

;;; Get display_name size
	Strlen dword [display]
	inc eax
	mov [displaystr_size],eax

;;; Alloc displaystr mem
	MemoryAllocate eax
	cmp eax,0
	jl near ConnectionError
	mov [displaystr],eax

;;; Copy display
	Strncpy dword [display], dword [displaystr], dword [displaystr_size]

;;;
;;; Extract protocol, hostname, display and screen
;;; 
	mov edi,[displaystr]
	mov ecx,[displaystr_size]
;;; Get protocol
	mov edx,edi
	mov ebx,ecx
	mov [protokoll],dword protokoll_str
	mov al,'/'
	cld
	repne scasb
	jecxz .noprotokoll
	mov [edi-1],byte 0
	mov [protokoll],edx
	mov edx,edi
	mov ebx,ecx
.noprotokoll:
	mov edi,edx
	mov ecx,ebx

;;; Get hostname
	mov [hostname],dword hostname_str
	mov al,':'
	cmp [edi],al
	je .nohostname

	cld
	repne scasb
	cmp ecx,0
	je near ConnectionError ; No ':'
	mov [edi-1],byte 0
	mov [hostname],edx
	mov edx,edi
	dec edx
	mov ebx,ecx
.nohostname:
	mov edi,edx
	mov ecx,ebx

;;; Get display nr and screen nr
	inc edi
	mov [display_nr],edi
	mov [screen_nr],dword screen_nr_str
	mov al,'.'
	cmp [edi],al
	je near ConnectionError
	cld
	repne scasb
	mov [edi-1],byte 0
	jecxz .noscreen_nr
	mov [screen_nr],edi
.noscreen_nr:

;;; 
;;; Make the connection
;;;

%ifdef ___VERBOSE_FLAG___
;;; First: Show what's up!
	mov eax,dword [_argv]
	WritefSTDOUT '%e: _XOpenDisplay: Connecting to: %e/%e:%e.%e',lf, dword [eax], dword [protokoll], dword [hostname], dword [display_nr], dword [screen_nr]
%endif	; ___VERBOSE_FLAG___

;;; Prepare the connection

;;; At this moment only: local/localhost:x.y
;;; -> Protokoll should be 'local' or 'unix' (both UNIX IPC)
;;; FIXME:XOpenDisplay.s: if hostname is given (without protocol) connection should be
;;;        established via TCP/IP
.isprot_local:
	mov eax,[protokoll]
	cmp [eax+0],dword 'loca'
	jne .isprot_unix
	cmp [eax+4],word 0x006c	; "l",0
	jne .isprot_unix
	jmp .ishostname_localhost

.isprot_unix:
	mov eax,[protokoll]
	cmp [eax+0],dword 'unix'
	jne near ConnectionError

;;; -> hostname should be 'localhost'
.ishostname_localhost:
	mov eax,[hostname]
	cmp [eax+0],dword 'loca'
	jne near ConnectionError
	cmp [eax+4],dword 'lhos'
	jne near ConnectionError
	cmp [eax+8],word 0x0074
	jne near ConnectionError

;;; Connect on local machine (UNIX IPC)
	call ConnectUNIX

;;; Convert [screen_nr]
	mov eax,[screen_nr]
	mov ecx,0
.conv_screen:
	movzx ebx,byte [eax]
	cmp ebx,0
	je .conv_screen_end
	sub ebx,48
	imul ecx,10
	add ecx,ebx
	inc eax
	jmp .conv_screen
.conv_screen_end:
	mov [screen_nr],ecx

;;; Free allocated memory
	MemoryFree dword [displaystr]
	mov [displaystr],dword 0

;;; Initiate Xprotokoll

	;; Send connection request
	mov [client+xConnClientPrefix.byteOrder],byte 'l' ; LSF
	mov [client+xConnClientPrefix.majorVersion],word X_PROTOCOL
	mov [client+xConnClientPrefix.minorVersion],word X_PROTOCOL_REVISION
	mov [client+xConnClientPrefix.nbytesAuthProto],word 0
	mov [client+xConnClientPrefix.nbytesAuthString],word 0
	lea eax,[client]
	sys_write dword [fd], eax, dword xConnClientPrefix_size
	cmp eax,0
	jl near ConnectionError

	;; Receive answer
.wait:
	Sleep dword 0, dword 50	; Wait for something?
	cmp eax,-EAGAIN
	je near .wait
	lea eax,[XSetupPrefix]
	sys_read dword [fd], eax, dword xConnSetupPrefix_size
	cmp eax,0
	jl near ConnectionError

	cmp [XSetupPrefix+xConnSetupPrefix.success],byte xTrue
	je near .xTrue

	cmp [XSetupPrefix+xConnSetupPrefix.success],byte xFalse
	jne .notxFalse
	mov eax,dword [_argv]
	WritefSTDERR '%e: _XOpenDisplay: Connection refused by server.',lf, dword [eax]
	jmp near ConnectionError

.notxFalse:
	;; H? Don't know what to do
	mov eax,dword [_argv]
	WritefSTDERR '%e: _XOpenDisplay: Unexpected connection setup replay from server, type %b',lf, dword [eax], dword [XSetupPrefix+xConnSetupPrefix.success]
	jmp ConnectionError

.xTrue:
	cmp [XSetupPrefix+xConnSetupPrefix.majorVersion],word X_PROTOCOL
	je .ProtOK

	;; Wrong protokoll!
	mov eax,dword [_argv]
	movzx ebx,word [XSetupPrefix+xConnSetupPrefix.majorVersion]
	movzx ecx,word [XSetupPrefix+xConnSetupPrefix.minorVersion]
	WritefSTDERR '%e: _XOpenDisplay: Client uses different protokoll version (%a.%a) than server (%a.%a)',lf, dword [eax], dword X_PROTOCOL, dword X_PROTOCOL_REVISION, ebx, ecx
	jmp near ConnectionError

.ProtOK:
	;; Allocate memory for additional serverinfo (returned to caller)
	movzx eax,word [XSetupPrefix+xConnSetupPrefix.length]
	shl eax,2
	MemoryAllocate eax
	cmp eax,0
	jl near ConnectionError
	mov [info],eax

	;; Read additional connection data
	movzx eax,word [XSetupPrefix+xConnSetupPrefix.length]
	shl eax,2
	sys_read dword [fd], dword [info], eax
	cmp eax,0
	jl near ConnectionError

;;; Setup the dpy-struc
	MemoryAllocate dword dpystruc_size
	cmp eax,0
	jl near ConnectionError
	mov [dpy],eax
	mov ebx,eax
	mov ecx,[info]

	mov eax,dword [fd]
	mov [ebx+dpystruc.fd],eax
	mov [ebx+dpystruc.info],ecx

	movzx eax,word [ecx+xConnSetup.maxRequestSize] ; maximum request size
	shl eax,2
	mov [ebx+dpystruc.maxRequestSize],eax ; in bytes

	mov [ebx+dpystruc.resource_id],dword 0
	mov eax,[ecx+xConnSetup.ridBase]
	mov [ebx+dpystruc.resource_base],eax
	mov eax,[ecx+xConnSetup.ridMask]
	mov [ebx+dpystruc.resource_mask],eax
	mov [ebx+dpystruc.resource_shift],dword 0
	push ecx
.SetupDpy1:
	bt eax,0
	jc .SetupDpy2
	inc dword [ebx+dpystruc.resource_shift]
	shr eax,1
	jmp short .SetupDpy1
.SetupDpy2:
	mov edx,[ebx+dpystruc.resource_mask]
	mov ecx,[ebx+dpystruc.resource_shift]
	shr edx,cl
	sub edx,5
	mov [ebx+dpystruc.resource_max],edx
	pop ecx

;;; X version
	movzx eax,word [XSetupPrefix+xConnSetupPrefix.majorVersion]
	mov [ebx+dpystruc.major_version],eax
	movzx eax,word [XSetupPrefix+xConnSetupPrefix.minorVersion]
	mov [ebx+dpystruc.minor_version],eax

;;; Image byte order
	movzx eax,byte [ecx+xConnSetup.imageByteOrder]
	mov [ebx+dpystruc.imageByteOrder],eax
;;; bitmap bit order
	movzx eax,byte [ecx+xConnSetup.bitmapBitOrder]
	mov [ebx+dpystruc.bitmapBitOrder],eax
;;; bitmap scanline unit
	movzx eax,byte [ecx+xConnSetup.bitmapScanlineUnit]
	mov [ebx+dpystruc.bitmapScanlineUnit],eax
;;; bitmap scanline pad
	movzx eax,byte [ecx+xConnSetup.bitmapScanlinePad]
	mov [ebx+dpystruc.bitmapScanlinePad],eax

;;; vendor string
	movzx eax,word [ecx+xConnSetup.nbytesVendor]
	mov [ebx+dpystruc.vendor_size],eax
	mov eax,ecx
	add eax,xConnSetup_size
	mov [ebx+dpystruc.vendor],eax

;;; pformats, nformats
	mov edx,[ebx+dpystruc.vendor_size]
	add edx,byte 3
	and edx,~(3)
	add eax,edx
	mov [ebx+dpystruc.pformats],eax
	movzx eax,byte [ecx+xConnSetup.numFormats]
	mov [ebx+dpystruc.nformats],eax

;;; pscreens, nscreens
	imul eax,xPixmapFormat_size
	add eax,[ebx+dpystruc.pformats]
	mov [ebx+dpystruc.pscreens],eax
	movzx eax,byte [ecx+xConnSetup.numRoots]
	mov [ebx+dpystruc.nscreens],eax

;;; Setup the standard error-handling-routine (null)
;;; may be modified by client
	mov [ebx+dpystruc.error_handler],dword 0
;;; Setup the standard event-handling-routine (eat 'em)
;;; may be modified by client
	mov [ebx+dpystruc.event_handler],dword 0

	mov eax,[screen_nr]
	mov [ebx+dpystruc.default_screen_nr],eax

	XScreenAddr ebx, eax
	cmp eax,-1
	je near ConnectionError
	mov [ebx+dpystruc.default_screen],eax
	mov ecx,eax		; ecx=default_screen ptr
	mov eax,[ecx+xWindowRoot.windowId]
	mov [ebx+dpystruc.default_root],eax
	movzx eax,byte [ecx+xWindowRoot.rootDepth]
	mov [ebx+dpystruc.default_depth],eax
	mov eax,[ecx+xWindowRoot.rootVisualID]
	mov [ebx+dpystruc.default_visualID],eax

	XFindVisualTypeDSAddr ebx, dword [ecx+xWindowRoot.rootVisualID], dword [ebx+dpystruc.default_depth], dword [ebx+dpystruc.default_screen_nr]
	cmp eax,-1
	je near ConnectionError
	mov [ebx+dpystruc.default_visualtype],eax
	mov eax,[ecx+xWindowRoot.defaultColormap]
	mov [ebx+dpystruc.default_colormap],eax

;;; XC-MISC Extension may be required
	mov [ebx+dpystruc.XC_MISC_opcode],dword 0
	lea eax,[ebx+dpystruc.XC_MISC_opcode]
	XQueryExtension ebx, dword XC_MISC, eax, dword 0, dword 0
	cmp eax,-1
	jne .XC_MISC_ok
	mov eax,dword [_argv]
	WritefSTDERR '%e: _XOpenDisplay: XC-MISC Extension may be required but is not present.',lf, dword [eax]
.XC_MISC_ok:
;;; MIT-SHM Extension may be required
	mov [ebx+dpystruc.MIT_SHM_opcode],dword 0
	lea eax,[ebx+dpystruc.MIT_SHM_opcode]
	XQueryExtension ebx, dword MIT_SHM, eax, dword 0, dword 0
	cmp eax,-1
	jne .MIT_SHM_ok
	mov eax,dword [_argv]
	WritefSTDERR '%e: _XOpenDisplay: MIT-SHM Extension may be required but is not present.',lf, dword [eax]
.MIT_SHM_ok:

;;; to be continued...



%ifdef ___VERBOSE_FLAG___
;;; Connection success!
	mov eax,dword [_argv]
	WritefSTDOUT '%e: _XOpenDisplay: Success.',lf, dword [eax]
%endif	; ___VERBOSE_FLAG___

;;; Return connection info
	mov eax,dword [dpy]

	__pop edi, esi, edx, ecx, ebx
	leave
	ret

ConnectionError:
	cmp [displaystr],dword 0
	jne .j1
	MemoryFree dword [displaystr]
.j1:
	cmp [info],dword 0
	je .j2
	MemoryFree dword [info]
.j2:
	cmp [dpy],dword 0
	je .j3
	MemoryFree dword [dpy]
.j3:
	cmp [fd],dword 0
	je .j4
	sys_close dword [fd]	; An error doesn't matter at this time
.j4:
	mov eax,dword [_argv]
	WritefSTDERR '%e: _XOpenDisplay: Could not connect to X server.',lf, dword [eax]
	sys_exit dword 1

;;;
;;; Connect on a local machine (via UNIX IPC)
;;; 
ConnectUNIX:
;;; Build pathname for connection node
	Strcpy dword [display_nr], dword UNIXNodeDisp

;;; Open Connection

;;; 1. Open a socket (UNIX IPC)
	Socket dword AF_UNIX, dword SOCK_STREAM, dword 0
	cmp eax,0
	jl near ConnectionError
	mov [fd],eax

;;; 2. Set to non-blocking mode
	sys_fcntl dword [fd], dword F_GETFL, dword 0
	cmp eax,0
	jl near ConnectionError
	or eax,O_NONBLOCK
	sys_fcntl dword [fd], dword F_SETFL, eax
	cmp eax,0
	jl near ConnectionError

;;; 3. Connect to X-socket
	Strlen dword UNIXNodeName
	add eax,byte 2
	Connect dword [fd], dword UNIX_AF, eax
	cmp eax,0
	jl near ConnectionError

;;; Yeah! We are connected to the X server... ?
	ret


	align 32
display_str:	db 'DISPLAY='
display_strlen	equ $-display_str
protokoll_str:	db 'unix',0
hostname_str:	db 'localhost',0
screen_nr_str:	db '0',0
XC_MISC:	db 'XC-MISC',0
MIT_SHM:	db 'MIT-SHM',0

	section .data
UNIX_AF:	dw AF_UNIX
UNIXNodeName:	db "/tmp/.X11-unix/X"
UNIXNodeDisp:	times 5 db 0


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: