;;
;;; XSync.s
;;;
;;; created: <Tue Dec 14 19:23:00 1999>
;;; last modified: <Fri Oct 25 13:33:13 2002>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; XSync: Flush buffer and check for replys, errors and events
;;;
;;; In: XSync dpy, *buf, len (dpy [eax+0], *buf [eax+4], len [eax+8])
;;; Out: eax=0 (on success) !=0 (reply: memory chunk)
;;;

%include "system.h"
%include "io.h"
%include "string.h"
%include "mem.h"
%include "x.h"
%include "sys.h"

global _XSync
extern _errno,_argv

%define dpy ebp+0
%define buf ebp+4
%define len ebp+8

;;; This needs ___VERBOSE_FLAG___ if ___DEBUG___ is wanted
%ifdef ___DEBUG___
 %define ___VERBOSE_FLAG___
%endif

%macro LOCKIT 0
%%lockloop:
	cmp [.lock],dword 0
	je %%nolock
	Sleep dword 0, dword 50
	jmp %%lockloop
%%nolock:
	mov [.lock],dword 1
%endm
%macro UNLOCKIT 0
	mov [.lock],dword 0
%endm

	section .text
	align 16
_XSync:
	__push ebx, ecx, ebp
	mov ebp,eax

	LOCKIT

	cmp [buf],dword 0
	je near .nothingtowrite
	cmp [len],dword 0
	je .nothingtowrite
;;; Update sequence number
	inc dword [.seq_number]
;;; Check if maximum request size is not exceeded
	mov eax,[dpy]
	mov eax,[eax+dpystruc.maxRequestSize]
	cmp [len],eax
	jbe .sizeok
;;; Request size error
	mov eax,.err1name
	jmp .err1
.sizeok:
	mov eax,[dpy]
	sys_write dword [eax+dpystruc.fd], dword [buf], dword [len]
	cmp eax,0
	jl near .err

%ifdef ___DEBUG___
	pushad
;;; 	WritefSTDERR 'DEBUG:XSync.s:%a: Send data.', lf, dword __LINE__
	mov esi,[buf]
	mov edx,0
	call .showtd
	popad
%endif	; ___DEBUG___

.nothingtowrite:
;;; Check for error/event/reply. If an error/event handler is defined, copy
;;; error/event into an allocated memory chunk and pass it to the handler.
;;; A reply is copied into a memory chunk and returned to the caller (in eax)
	mov ebx,[dpy]
	mov ecx,32
.readloop:
	Sleep dword 0, dword 50	; Wait for something?
	mov eax,.reply_buf+32
	sub eax,ecx
.readloop2:
	sys_read dword [ebx+dpystruc.fd], eax, ecx
	cmp eax,-EAGAIN		; No data for reading
	je near .end
	cmp eax,-EINTR
	je near .readloop2
	cmp eax,0
	jl near .err
	je .ready
	cmp eax,ecx
	jnb .ready
	sub ecx,eax
	jmp short .readloop
.ready:
;;; Error/event/reply?
	cmp [.reply_buf+0],byte X_Error
	je near .is_error
	cmp [.reply_buf+0],byte X_Reply
	ja near .is_event
%ifdef ___DEBUG___
	pushad
	movzx esi,word [.reply_buf+2]
	WritefSTDERR 'DEBUG:XSync.s:Reply(%a)', lf, esi
	mov esi,.reply_buf
	mov edx,1
	call .showtd
	popad
%endif	; ___DEBUG___
;;; It is a reply ([.reply_buf+0] = 1)
	mov eax,[.reply_buf+4]	; get length*4
	shl eax,2
	add eax,32
	MemoryAllocate eax
	cmp eax,-1
	je near .err0
	push eax
	mov ebx,eax
	Strncpy dword .reply_buf, eax, dword 32	; Using this macro will change eax!
	add ebx,32
;;; Read the rest of that reply
.reply_readloop:
	mov ecx,[.reply_buf+4]	; get length*4
	shl ecx,2
.reply_readloop2:
	mov eax,[dpy]
	sys_read dword [eax+dpystruc.fd], ebx, ecx
	cmp eax,-EAGAIN		; No data for reading
	je near .err
	cmp eax,-EINTR
	je near .reply_readloop2
	cmp eax,0
	jl near .err		; jle
	je .reply_ready
	cmp eax,ecx
	jnb .reply_ready
	add ebx,eax
	sub ecx,eax
	jmp .readloop
.reply_ready:
	pop eax
	jmp .end2		; return memory chunk

;;; It is an event
.is_event:
%ifdef ___DEBUG___
	pushad
	movzx esi,word [.reply_buf+2]
	WritefSTDERR 'DEBUG:XSync.s:Event(%a)', lf, esi
	mov esi,.reply_buf
	mov edx,1
	call .showtd
	popad
%endif	; ___DEBUG___
	mov eax,[dpy]
	cmp [eax+dpystruc.event_handler],dword 0
	je near .std_event_handler
;;; Pass it to the specified handler (dpy [eax+0], mem [eax+4])
;;; push dword [mem]
;;; push dword [dpy]
;;; mov eax,esp
;;; call [handler]
;;; add esp,byte 8
	MemoryAllocate dword 32
	cmp eax,-1
	je near .err0
	push eax
	push dword [dpy]
	Strncpy dword .reply_buf, eax, dword 32	; Using this macro will change eax!
	mov ebx,[dpy]
	mov ebx,[ebx+dpystruc.event_handler]
	UNLOCKIT
	mov eax,esp
	call ebx
	LOCKIT
	add esp,byte 4		; Throw away dpy on stack
	pop eax			; Free chunk
	MemoryFree eax
	jmp .nothingtowrite	; Get all of them

.std_event_handler:
; eat 'em!
%ifdef ___DEBUG___
	pushad
	movzx esi,word [.reply_buf+2]
	WritefSTDERR 'DEBUG:XSync.s:Event(%a) has been eaten!', lf, esi
	mov esi,.reply_buf
	mov edx,1
	call .showtd
	popad
%endif	; ___DEBUG___
	jmp .nothingtowrite
;;; 	ret

;;; Whoa! This is an error!
.is_error:
%ifdef ___DEBUG___
	pushad
	movzx esi,word [.reply_buf+2]
	WritefSTDERR 'DEBUG:XSync.s:Error(%a)', lf, esi
	mov esi,.reply_buf
	mov edx,1
	call .showtd
	popad
%endif	; ___DEBUG___
;;; But: If code is 0...
;;;	cmp [.reply_buf+1],byte 0	;;; FIXME:XSync.s: Is this correct?
;;;	je near .end		; it is not an error!!!
	mov eax,[dpy]
	cmp [eax+dpystruc.error_handler],dword 0
	je near .std_err_handler
;;; Pass it to the specified handler (dpy [eax+0], mem [eax+4])
;;; push dword [mem]
;;; push dword [dpy]
;;; mov eax,esp
;;; call [handler]
;;; add esp,byte 8
	MemoryAllocate dword 32
	cmp eax,-1
	je near .err0
	push eax
	push dword [dpy]
	Strncpy dword .reply_buf, eax, dword 32	; Using this macro will change eax!
	mov ebx,[dpy]
	mov ebx,[ebx+dpystruc.error_handler]
	UNLOCKIT
	mov eax,esp
	call ebx
	LOCKIT
	add esp,byte 4		; Throw away dpy on stack
	pop eax			; Free chunk
	MemoryFree eax
	jmp .nothingtowrite	; Get all of them

.std_err_handler:
;;; Give a message and exit(1)
	mov edi,.reply_buf
	mov ebx,[dpy]

%ifdef ___VERBOSE_FLAG___
;;; Find func name
	movzx eax,byte [edi+10]
	cmp eax,[ebx+dpystruc.MIT_SHM_opcode] ; May be...
	je near .std_err_handler_MIT_SHM1
	cmp eax,[ebx+dpystruc.XC_MISC_opcode] ; or...
	je near .std_err_handler_XC_MISC1

	mov [.ErrInMinFunc],dword .ExtErr
	cmp eax,X_NoOperation
	jna .std_err_handler_iir0
	mov [.ErrInFunc],dword .ExtFunc
	jmp near .std_err_handler_wrt0
.std_err_handler_iir0:
	mov esi,.FuncNames-1
.std_err_handler_scan0:
	add esi,byte 5
	cmp [esi-4],eax
	jne .std_err_handler_scan1
	mov [.ErrInFunc],esi
	jmp .std_err_handler_wrt0
.std_err_handler_scan1:
	inc esi
	cmp [esi],byte 0
	jne .std_err_handler_scan1
	jmp .std_err_handler_scan0

.std_err_handler_MIT_SHM1:	; For MIT-SHM
	mov [.ErrInFunc],dword .MIT_SHMFunc
	movzx eax,word [edi+8]
	cmp eax,X_ShmCreatePixmap
	jna .std_err_handler_MIT_SHM2
	mov [.ErrInMinFunc],dword .ExtFunc
	jmp .std_err_handler_wrt0
.std_err_handler_MIT_SHM2:
	mov esi,.MIT_SHMFuncNames-1
.std_err_handler_MIT_SHM3:
	add esi,byte 5
	cmp [esi-4],eax
	jne .std_err_handler_MIT_SHM4
	mov [.ErrInMinFunc],esi
	jmp .std_err_handler_wrt0
.std_err_handler_MIT_SHM4:
	inc esi
	cmp [esi],byte 0
	jne .std_err_handler_MIT_SHM4
	jmp .std_err_handler_MIT_SHM3

	jmp .std_err_handler_wrt0

.std_err_handler_XC_MISC1:	; For XC-MISC
	mov [.ErrInFunc],dword .XC_MISCFunc
	movzx eax,word [edi+8]
	cmp eax,X_XCMiscGetXIDList
	jna .std_err_handler_XC_MISC2
	mov [.ErrInMinFunc],dword .ExtFunc
	jmp short .std_err_handler_wrt0
.std_err_handler_XC_MISC2:
	mov esi,.XC_MISCFuncNames-1
.std_err_handler_XC_MISC3:
	add esi,byte 5
	cmp [esi-4],eax
	jne .std_err_handler_XC_MISC4
	mov [.ErrInMinFunc],esi
	jmp .std_err_handler_wrt0
.std_err_handler_XC_MISC4:
	inc esi
	cmp [esi],byte 0
	jne .std_err_handler_XC_MISC4
	jmp .std_err_handler_XC_MISC3

.std_err_handler_wrt0:

;;; Find error name
	movzx eax,byte [edi+1]
	cmp eax,17
	jna .std_err_handler_iir
	mov esi,.ExtErr
	jmp short .std_err_handler_wrt
.std_err_handler_iir:
	mov esi,.ErrorNames-1
.std_err_handler_scan:
	inc esi
	cmp [esi],byte 0
	jne .std_err_handler_scan
	dec eax
	jnz .std_err_handler_scan
	inc esi
.std_err_handler_wrt:
%endif	; ___VERBOSE_FLAG___

	movzx eax,byte [edi+1]
	movzx ebx,word [edi+2]
	mov ecx,[edi+4]
	movzx edx,word [edi+8]
	movzx ebp,byte [edi+10]
	mov edi,dword [_argv]

%ifndef ___VERBOSE_FLAG___
	WritefSTDERR '%e: _XSync: Received error: code = %a, sequence number = %a, value = %d, minor opcode = %a, major opcode = %a.',lf, dword [edi], eax, ebx, ecx, edx, ebp
%endif	; ___VERBOSE_FLAG___
%ifdef ___VERBOSE_FLAG___
	WritefSTDERR '%e: _XSync: Received error: code = %a (%e), sequence number = %a, value = %d, minor opcode = %a (%e), major opcode = %a (%e).',lf, dword [edi], eax, esi, ebx, ecx, edx, dword [.ErrInMinFunc], ebp, dword [.ErrInFunc]
%endif	; ___VERBOSE_FLAG___

;;; Only the current process failed, so unlock here
	UNLOCKIT
	sys_exit 1

.end:
	mov eax,0
.end2:
	UNLOCKIT
	__pop ebp, ecx, ebx
	ret

;;; Only the current process failed, so unlock here
.err0:
	mov eax,[_errno]
.err:
	UNLOCKIT
	GetErrorStr eax
.err1:				; jmp for request size error
	mov ebx,dword [_argv]
	WritefSTDERR '%e: _XSync: Could not send to/receive from Xserver (%e).',lf, dword [ebx], eax
	sys_exit 1

;;; Show transmitted datas:
;;; Should be called with backupped registers
;;; in: esi=buffer, edx=dataflow(1:in,else:out)
%ifdef ___DEBUG___
%define width 10
.showtd:
	cmp edx,1
	jne .showtd_isout
	mov edx,32		; length
	jmp .showtd_doit

.showtd_isout:

;;; Give a request name
	mov ebx,[dpy]
;;; Find func name
	movzx eax,byte [esi]
	cmp eax,[ebx+dpystruc.MIT_SHM_opcode] ; May be...
	je near .showtd_MIT_SHM
	cmp eax,[ebx+dpystruc.XC_MISC_opcode] ; or...
	je near .showtd_XC_MISC

	mov edi,.FuncNames-1
.showtd_scan0:
	add edi,byte 5
	cmp [edi-4],eax
	jne .showtd_scan1
	jmp .showtd_wrt0
.showtd_scan1:
	inc edi
	cmp [edi],byte 0
	jne .showtd_scan1
	jmp short .showtd_scan0

.showtd_MIT_SHM:
	mov edi,.MIT_SHMFunc
	jmp short .showtd_wrt0

.showtd_XC_MISC:
	mov edi,.XC_MISCFunc
	jmp short .showtd_wrt0

.showtd_wrt0:
	WritefSTDERR 'DEBUG:XSync.s:Request(%a): %e',lf, dword [.seq_number], edi

	movzx edx,word [esi+2]	; length
	shl edx,2

.showtd_doit:
	WritefSTDERR 'DEBUG:XSync.s:DATA: '
.showtd_outerloop:
	mov ecx,width
.showtd_innerloop:
	WritefSTDERR '%b ', dword [esi]
	inc esi
	dec edx
	jz .showtd_end
	loop .showtd_innerloop
	WritefSTDERR lf, 'DEBUG:XSync.s:DATA: '
	jmp short .showtd_outerloop

.showtd_end:
	WritefSTDERR lf
	ret

%endif	; ___DEBUG___

	align 32
.err1name:
	db 'maximum request size exceeded',0
%ifdef ___VERBOSE_FLAG___
.ErrorNames:
	db 0
	db 'Request',0
	db 'Value',0
	db 'Window',0
	db 'Pixmap',0
	db 'Atom',0
	db 'Cursor',0
	db 'Font',0
	db 'Match',0
	db 'Drawable',0
	db 'Access',0
	db 'Alloc',0
	db 'Colormap',0
	db 'GContext',0
	db 'IDChoice',0
	db 'Name',0
	db 'Length',0
	db 'Implementation',0
.ExtErr:
	db 'unknown',0

.FuncNames:
	dd X_CreateWindow
	db 'CreateWindow',0
	dd X_ChangeWindowAttributes
	db 'ChangeWindowAttributes',0
	dd X_GetWindowAttributes
	db 'GetWindowAttributes',0
	dd X_DestroyWindow
	db 'DestroyWindow',0
	dd X_DestroySubwindows
	db 'DestroySubwindows',0
	dd X_ChangeSaveSet
	db 'ChangeSaveSet',0
	dd X_ReparentWindow
	db 'ReparentWindow',0
	dd X_MapWindow
	db 'MapWindow',0
	dd X_MapSubwindows
	db 'MapSubwindows',0
	dd X_UnmapWindow
	db 'UnmapWindow',0
	dd X_UnmapSubwindows
	db 'UnmapSubwindows',0
	dd X_ConfigureWindow
	db 'ConfigureWindow',0
	dd X_CirculateWindow
	db 'CirculateWindow',0
	dd X_GetGeometry
	db 'GetGeometry',0
	dd X_QueryTree
	db 'QueryTree',0
	dd X_InternAtom
	db 'InternAtom',0
	dd X_GetAtomName
	db 'GetAtomName',0
	dd X_ChangeProperty
	db 'ChangeProperty',0
	dd X_DeleteProperty
	db 'DeleteProperty',0
	dd X_GetProperty
	db 'GetProperty',0
	dd X_ListProperties
	db 'ListProperties',0
	dd X_SetSelectionOwner
	db 'SetSelectionOwner',0
	dd X_GetSelectionOwner
	db 'GetSelectionOwner',0
	dd X_ConvertSelection
	db 'ConvertSelection',0
	dd X_SendEvent
	db 'SendEvent',0
	dd X_GrabPointer
	db 'GrabPointer',0
	dd X_UngrabPointer
	db 'UngrabPointer',0
	dd X_GrabButton
	db 'GrabButton',0
	dd X_UngrabButton
	db 'UngrabButton',0
	dd X_ChangeActivePointerGrab
	db 'ChangeActivePointerGrab',0
	dd X_GrabKeyboard
	db 'GrabKeyboard',0
	dd X_UngrabKeyboard
	db 'UngrabKeyboard',0
	dd X_GrabKey
	db 'GrabKey',0
	dd X_UngrabKey
	db 'UngrabKey',0
	dd X_AllowEvents
	db 'AllowEvents',0
	dd X_GrabServer
	db 'GrabServer',0
	dd X_UngrabServer
	db 'UngrabServer',0
	dd X_QueryPointer
	db 'QueryPointer',0
	dd X_GetMotionEvents
	db 'GetMotionEvents',0
	dd X_TranslateCoords
	db 'TranslateCoords',0
	dd X_WarpPointer
	db 'WarpPointer',0
	dd X_SetInputFocus
	db 'SetInputFocus',0
	dd X_GetInputFocus
	db 'GetInputFocus',0
	dd X_QueryKeymap
	db 'QueryKeymap',0
	dd X_OpenFont
	db 'OpenFont',0
	dd X_CloseFont
	db 'CloseFont',0
	dd X_QueryFont
	db 'QueryFont',0
	dd X_QueryTextExtents
	db 'QueryTextExtents',0
	dd X_ListFonts
	db 'ListFonts',0
	dd X_ListFontsWithInfo
	db 'ListFontsWithInfo',0
	dd X_SetFontPath
	db 'SetFontPath',0
	dd X_GetFontPath
	db 'GetFontPath',0
	dd X_CreatePixmap
	db 'CreatePixmap',0
	dd X_FreePixmap
	db 'FreePixmap',0
	dd X_CreateGC
	db 'CreateGC',0
	dd X_ChangeGC
	db 'ChangeGC',0
	dd X_CopyGC
	db 'CopyGC',0
	dd X_SetDashes
	db 'SetDashes',0
	dd X_SetClipRectangles
	db 'SetClipRectangles',0
	dd X_FreeGC
	db 'FreeGC',0
	dd X_ClearArea
	db 'ClearArea',0
	dd X_CopyArea
	db 'CopyArea',0
	dd X_CopyPlane
	db 'CopyPlane',0
	dd X_PolyPoint
	db 'PolyPoint',0
	dd X_PolyLine
	db 'PolyLine',0
	dd X_PolySegment
	db 'PolySegment',0
	dd X_PolyRectangle
	db 'PolyRectangle',0
	dd X_PolyArc
	db 'PolyArc',0
	dd X_FillPoly
	db 'FillPoly',0
	dd X_PolyFillRectangle
	db 'PolyFillRectangle',0
	dd X_PolyFillArc
	db 'PolyFillArc',0
	dd X_PutImage
	db 'PutImage',0
	dd X_GetImage
	db 'GetImage',0
	dd X_PolyText8
	db 'PolyText8',0
	dd X_PolyText16
	db 'PolyText16',0
	dd X_ImageText8
	db 'ImageText8',0
	dd X_ImageText16
	db 'ImageText16',0
	dd X_CreateColormap
	db 'CreateColormap',0
	dd X_FreeColormap
	db 'FreeColormap',0
	dd X_CopyColormapAndFree
	db 'CopyColormapAndFree',0
	dd X_InstallColormap
	db 'InstallColormap',0
	dd X_UninstallColormap
	db 'UninstallColormap',0
	dd X_ListInstalledColormaps
	db 'ListInstalledColormaps',0
	dd X_AllocColor
	db 'AllocColor',0
	dd X_AllocNamedColor
	db 'AllocNamedColor',0
	dd X_AllocColorCells
	db 'AllocColorCells',0
	dd X_AllocColorPlanes
	db 'AllocColorPlanes',0
	dd X_FreeColors
	db 'FreeColors',0
	dd X_StoreColors
	db 'StoreColors',0
	dd X_StoreNamedColor
	db 'StoreNamedColor',0
	dd X_QueryColors
	db 'QueryColors',0
	dd X_LookupColor
	db 'LookupColor',0
	dd X_CreateCursor
	db 'CreateCursor',0
	dd X_CreateGlyphCursor
	db 'CreateGlyphCursor',0
	dd X_FreeCursor
	db 'FreeCursor',0
	dd X_RecolorCursor
	db 'RecolorCursor',0
	dd X_QueryBestSize
	db 'QueryBestSize',0
	dd X_QueryExtension
	db 'QueryExtension',0
	dd X_ListExtensions
	db 'ListExtensions',0
	dd X_ChangeKeyboardMapping
	db 'ChangeKeyboardMapping',0
	dd X_GetKeyboardMapping
	db 'GetKeyboardMapping',0
	dd X_ChangeKeyboardControl
	db 'ChangeKeyboardControl',0
	dd X_GetKeyboardControl
	db 'GetKeyboardControl',0
	dd X_Bell
	db 'Bell',0
	dd X_ChangePointerControl
	db 'ChangePointerControl',0
	dd X_GetPointerControl
	db 'GetPointerControl',0
	dd X_SetScreenSaver
	db 'SetScreenSaver',0
	dd X_GetScreenSaver
	db 'GetScreenSaver',0
	dd X_ChangeHosts
	db 'ChangeHosts',0
	dd X_ListHosts
	db 'ListHosts',0
	dd X_SetAccessControl
	db 'SetAccessControl',0
	dd X_SetCloseDownMode
	db 'SetCloseDownMode',0
	dd X_KillClient
	db 'KillClient',0
	dd X_RotateProperties
	db 'RotateProperties',0
	dd X_ForceScreenSaver
	db 'ForceScreenSaver',0
	dd X_SetPointerMapping
	db 'SetPointerMapping',0
	dd X_GetPointerMapping
	db 'GetPointerMapping',0
	dd X_SetModifierMapping
	db 'SetModifierMapping',0
	dd X_GetModifierMapping
	db 'GetModifierMapping',0
	dd X_NoOperation
	db 'NoOperation',0
.ExtFunc:
	db 'extended',0
.MIT_SHMFunc:
	db 'MIT-SHM',0
.MIT_SHMFuncNames:
	dd X_ShmQueryVersion
	db 'ShmQueryVersion',0
	dd X_ShmAttach
	db 'ShmAttach',0
	dd X_ShmDetach
	db 'ShmDetach',0
	dd X_ShmPutImage
	db 'ShmPutImage',0
	dd X_ShmGetImage
	db 'ShmGetImage',0
	dd X_ShmCreatePixmap
	db 'ShmCreatePixmap',0
.XC_MISCFunc:
	db 'XC-MISC',0
.XC_MISCFuncNames:
	dd X_XCMiscGetVersion
	db 'XCMiscGetVersion',0
	dd X_XCMiscGetXIDRange
	db 'XCMiscGetXIDRange',0
	dd X_XCMiscGetXIDList
	db 'XCMiscGetXIDList',0
%endif	; ___VERBOSE_FLAG___

	section .data
.lock:		dd 0
.seq_number:	resd 1
	section .bss
%ifdef ___VERBOSE_FLAG___
.ErrInFunc:	resd 1
.ErrInMinFunc:	resd 1
%endif	; ___VERBOSE_FLAG___
.reply_buf:	resb 32


;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: