;;;
;;; UnzipBZIP2.s
;;;
;;; created: <Tue May 25 15:54:36 1999>
;;; last modified: <Mon Apr 10 01:25:52 2000>
;;;
;;; Christian Fowelin <christian@fowelin.de>
;;;
;;; Released under the LGPL (file COPYING).
;;;

;;;
;;; Bunzips n buf_in to buf_out
;;; In: UnzipBZIP2 buf_in, n (*buf_in [eax+0], n [eax+4])
;;; Out: eax=buf_out
;;;

;;;
;;; buf_out:
;;; 0 dword new size
;;; 4 [dword 0] bytes: data
;;;
%include "system.h"
%include "sys.h"
%include "ipc.h"
%include "mem.h"

global _UnzipBZIP2
extern _envp,_errno

%define _buf_in ebx+0
%define _quelle_length ebx+4

%define buf_in ebp-4
%define quelle_length ebp-8
%define buf_out ebp-12
%define fd ebp-20
%define buf_out_length ebp-24
%define buf_out_read ebp-28
%define buf_pos ebp-32


	section .text
	align 32
_UnzipBZIP2:
	push ebp
	mov ebp,esp
	sub esp,byte 32
	__push ebx, ecx, edx, esi, edi
	mov ebx,eax
	mov eax,[_buf_in]
	mov [buf_in],eax
	mov eax,[_quelle_length]
	mov [quelle_length],eax

	;; alloc mem for buf_out
	cmp [quelle_length],dword 1024
	jnb .alloc
	MemoryAllocate dword 1024
	jmp short .alloc2
.alloc:
	MemoryAllocate dword [quelle_length]
.alloc2:
	cmp eax,0
	jl near .err
	mov [buf_out],eax
	add eax,byte 4
	mov [buf_pos],eax

	;; Setup the pipe
	lea eax,[fd]		; eax=ebp-12
	PipeIo eax, 'bzip2', '-d'

	;; fork child
	sys_fork
	cmp eax,0
	jl near .err
	jne near .parent

	;; child
	sys_close dword [fd+0]
	sys_write dword [fd+4], dword [buf_in], dword [quelle_length]
	sys_close dword [fd+4]
	sys_exit

.parent:
	sys_close dword [fd+4]
	mov [buf_out_read],dword 0
.loop:
	mov eax,[buf_out_length]
	sub eax,[buf_out_read]
	cmp eax,1024
	jae .read
	add [buf_out_length],dword 1024*10
	MemoryResize dword [buf_out], dword [buf_out_length]
	cmp eax,0
	jl near .err

.read:
	sys_read dword [fd+0], dword [buf_pos], dword 1024
	cmp eax,0
	jl .err
	add [buf_pos],eax
	add [buf_out_read],eax
	cmp eax,1024
	je .loop

	MemoryResize dword [buf_out], dword [buf_out_read]
	cmp eax,0
	jl near .err

	mov ebx,[buf_out]
	mov eax,[buf_out_read]
	sub eax,byte 4
	mov [ebx],eax

	sys_close dword [fd+0]

	mov eax,[buf_out]
	
	__pop edi, esi, edx, ecx, ebx
	leave
	ret

.err:
	mov [_errno],eax
	mov eax,-1
	__pop edi, esi, edx, ecx, ebx
	leave
	ret

;;; Local Variables:
;;;  mode:asm
;;;  eval:(load-library "time-stamp")
;;;  eval:(make-local-variable 'write-file-hooks)
;;;  eval:(add-hook 'write-file-hooks 'time-stamp)
;;;  eval:(setq time-stamp-start "last modified:[ 	]+\\\\?[\"<]+")
;;;  eval:(setq time-stamp-format "%3a %3b %2d %02H:%02M:%02S %y")
;;; End: