﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Drawing;

namespace VizBin
{
    internal interface ImageProcessor
    {
        /// <summary>
        /// Return an Bitmap given a byte array, width, and height of the image. See the getImage method
        /// for the checks that will have been performed on the arguments.
        /// </summary>
        /// <param name="vab_bytes">byte[] array</param>
        /// <param name="vn_width">width</param>
        /// <param name="vn_height">height</param>
        /// <returns>Bitmap</returns>
        Bitmap createImage(byte[] vab_bytes, int vn_width, int vn_height);

		/// <summary>
		/// Used to create an "overview" of the full file instead of just a portion. The intent of this
		/// method is to function just like the createImage method but without painting the image at the
		/// bit detail level and stick with the byte detail level.
		/// </summary>
		/// <param name="vab_bytes">byte[] array</param>
		/// <param name="vn_width">width</param>
		/// <param name="vn_height">height</param>
		/// <returns>Bitmap</returns>
		Bitmap createOverviewImage(byte[] vab_bytes, int vn_width, int vn_height);

        /// <summary>
        /// Returns the name of ImageProcessor. Should be a descriptive name as this will be presented to
        /// the user.
        /// </summary>
        /// <returns>String name of the processor</returns>
        String getName();
    }
}
